#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Manage user accounts on a Google Compute Engine instances."""

import datetime
import json
import logging.handlers
import optparse
import random

from google_compute_engine import config_manager
from google_compute_engine import constants
from google_compute_engine import file_utils
from google_compute_engine import logger
from google_compute_engine import metadata_watcher
from google_compute_engine.accounts import accounts_utils
from google_compute_engine.accounts import oslogin_utils

LOCKFILE = constants.LOCALSTATEDIR + '/lock/google_accounts.lock'


class AccountsDaemon(object):
  """Manage user accounts based on changes to metadata."""

  invalid_users = set()
  user_ssh_keys = {}

  def __init__(
      self, groups=None, remove=False, gpasswd_add_cmd=None,
      gpasswd_remove_cmd=None, groupadd_cmd=None, useradd_cmd=None,
      userdel_cmd=None, usermod_cmd=None, debug=False):
    """Constructor.

    Args:
      groups: string, a comma separated list of groups.
      remove: bool, True if deprovisioning a user should be destructive.
      useradd_cmd: string, command to create a new user.
      userdel_cmd: string, command to delete a user.
      usermod_cmd: string, command to modify user's groups.
      groupadd_cmd: string, command to add a new group.
      gpasswd_add_cmd: string, command to add an user to a group.
      gpasswd_remove_cmd: string, command to remove an user from a group.
      debug: bool, True if debug output should write to the console.
    """
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='google-accounts', debug=debug, facility=facility)
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)
    self.utils = accounts_utils.AccountsUtils(
        logger=self.logger, groups=groups, remove=remove,
        gpasswd_add_cmd=gpasswd_add_cmd, gpasswd_remove_cmd=gpasswd_remove_cmd,
        groupadd_cmd=groupadd_cmd, useradd_cmd=useradd_cmd,
        userdel_cmd=userdel_cmd, usermod_cmd=usermod_cmd)
    self.oslogin = oslogin_utils.OsLoginUtils(logger=self.logger)

    try:
      with file_utils.LockFile(LOCKFILE):
        self.logger.info('Starting Google Accounts daemon.')
        timeout = 60 + random.randint(0, 30)
        self.watcher.WatchMetadata(
            self.HandleAccounts, recursive=True, timeout=timeout)
    except (IOError, OSError) as e:
      self.logger.warning(str(e))

  def _HasExpired(self, key):
    """Check whether an SSH key has expired.

    Uses Google-specific semantics of the OpenSSH public key format's comment
    field to determine if an SSH key is past its expiration timestamp, and
    therefore no longer to be trusted. This format is still subject to change.
    Reliance on it in any way is at your own risk.

    Args:
      key: string, a single public key entry in OpenSSH public key file format.
          This will be checked for Google-specific comment semantics, and if
          present, those will be analysed.

    Returns:
      bool, True if the key has Google-specific comment semantics and has an
          expiration timestamp in the past, or False otherwise.
    """
    self.logger.debug('Processing key: %s.', key)

    try:
      schema, json_str = key.split(None, 3)[2:]
    except (ValueError, AttributeError):
      self.logger.debug('No schema identifier. Not expiring key.')
      return False

    if schema != 'google-ssh':
      self.logger.debug('Invalid schema %s. Not expiring key.', schema)
      return False

    try:
      json_obj = json.loads(json_str)
    except ValueError:
      self.logger.debug('Invalid JSON %s. Not expiring key.', json_str)
      return False

    if 'expireOn' not in json_obj:
      self.logger.debug('No expiration timestamp. Not expiring key.')
      return False

    expire_str = json_obj['expireOn']
    format_str = '%Y-%m-%dT%H:%M:%S+0000'
    try:
      expire_time = datetime.datetime.strptime(expire_str, format_str)
    except ValueError:
      self.logger.warning(
          'Expiration timestamp "%s" not in format %s. Not expiring key.',
          expire_str, format_str)
      return False

    # Expire the key if and only if we have exceeded the expiration timestamp.
    return datetime.datetime.utcnow() > expire_time

  def _ParseAccountsData(self, account_data):
    """Parse the SSH key data into a user map.

    Args:
      account_data: string, the metadata server SSH key attributes data.

    Returns:
      dict, a mapping of the form: {'username': ['sshkey1, 'sshkey2', ...]}.
    """
    if not account_data:
      return {}
    lines = [line for line in account_data.splitlines() if line]
    user_map = {}
    for line in lines:
      if not all(ord(c) < 128 for c in line):
        self.logger.info('SSH key contains non-ascii character: %s.', line)
        continue
      split_line = line.split(':', 1)
      if len(split_line) != 2:
        self.logger.info('SSH key is not a complete entry: %s.', split_line)
        continue
      user, key = split_line
      if self._HasExpired(key):
        self.logger.debug('Expired SSH key for user %s: %s.', user, key)
        continue
      if user not in user_map:
        user_map[user] = []
      user_map[user].append(key)
    logging.debug('User accounts: %s.', user_map)
    return user_map

  def _GetInstanceAndProjectAttributes(self, metadata_dict):
    """Get dictionaries for instance and project attributes.

    Args:
      metadata_dict: json, the deserialized contents of the metadata server.

    Returns:
      tuple, two dictionaries for instance and project attributes.
    """
    metadata_dict = metadata_dict or {}

    try:
      instance_data = metadata_dict['instance']['attributes']
    except KeyError:
      instance_data = {}
      self.logger.warning('Instance attributes were not found.')

    try:
      project_data = metadata_dict['project']['attributes']
    except KeyError:
      project_data = {}
      self.logger.warning('Project attributes were not found.')

    return instance_data, project_data

  def _GetAccountsData(self, metadata_dict):
    """Get the user accounts specified in metadata server contents.

    Args:
      metadata_dict: json, the deserialized contents of the metadata server.

    Returns:
      dict, a mapping of the form: {'username': ['sshkey1, 'sshkey2', ...]}.
    """
    instance_data, project_data = self._GetInstanceAndProjectAttributes(
        metadata_dict)
    valid_keys = [instance_data.get('sshKeys'), instance_data.get('ssh-keys')]
    block_project = instance_data.get('block-project-ssh-keys', '').lower()
    if block_project != 'true' and not instance_data.get('sshKeys'):
      valid_keys.append(project_data.get('ssh-keys'))
      valid_keys.append(project_data.get('sshKeys'))
    accounts_data = '\n'.join([key for key in valid_keys if key])
    return self._ParseAccountsData(accounts_data)

  def _UpdateUsers(self, update_users):
    """Provision and update Linux user accounts based on account metadata.

    Args:
      update_users: dict, authorized users mapped to their public SSH keys.
    """
    for user, ssh_keys in update_users.items():
      if not user or user in self.invalid_users:
        continue
      configured_keys = self.user_ssh_keys.get(user, [])
      if set(ssh_keys) != set(configured_keys):
        if not self.utils.UpdateUser(user, ssh_keys):
          self.invalid_users.add(user)
        else:
          self.user_ssh_keys[user] = ssh_keys[:]

  def _RemoveUsers(self, remove_users):
    """Deprovision Linux user accounts that do not appear in account metadata.

    Args:
      remove_users: list, the username strings of the Linux accounts to remove.
    """
    for username in remove_users:
      self.utils.RemoveUser(username)
      self.user_ssh_keys.pop(username, None)
    self.invalid_users -= set(remove_users)

  def _GetEnableOsLoginValue(self, metadata_dict):
    """Get the value of the enable-oslogin metadata key.

    Args:
      metadata_dict: json, the deserialized contents of the metadata server.

    Returns:
      bool, True if OS Login is enabled for VM access.
    """
    instance_data, project_data = self._GetInstanceAndProjectAttributes(
        metadata_dict)
    instance_value = instance_data.get('enable-oslogin')
    project_value = project_data.get('enable-oslogin')
    value = instance_value or project_value or ''

    return value.lower() == 'true'

  def _GetEnableTwoFactorValue(self, metadata_dict):
    """Get the value of the enable-oslogin-2fa metadata key.

    Args:
      metadata_dict: json, the deserialized contents of the metadata server.

    Returns:
      bool, True if two factor authentication is enabled for VM access.
    """
    instance_data, project_data = self._GetInstanceAndProjectAttributes(
        metadata_dict)
    instance_value = instance_data.get('enable-oslogin-2fa')
    project_value = project_data.get('enable-oslogin-2fa')
    value = instance_value or project_value or ''

    return value.lower() == 'true'

  def HandleAccounts(self, result):
    """Called when there are changes to the contents of the metadata server.

    Args:
      result: json, the deserialized contents of the metadata server.
    """
    self.logger.debug('Checking for changes to user accounts.')
    configured_users = self.utils.GetConfiguredUsers()
    enable_oslogin = self._GetEnableOsLoginValue(result)
    enable_two_factor = self._GetEnableTwoFactorValue(result)
    if enable_oslogin:
      desired_users = {}
      self.oslogin.UpdateOsLogin(True, two_factor_desired=enable_two_factor)
    else:
      desired_users = self._GetAccountsData(result)
      self.oslogin.UpdateOsLogin(False)
    remove_users = sorted(set(configured_users) - set(desired_users.keys()))
    self._UpdateUsers(desired_users)
    self._RemoveUsers(remove_users)
    self.utils.SetConfiguredUsers(desired_users.keys())


def main():
  parser = optparse.OptionParser()
  parser.add_option(
      '-d', '--debug', action='store_true', dest='debug',
      help='print debug output to the console.')
  (options, _) = parser.parse_args()
  instance_config = config_manager.ConfigManager()
  if instance_config.GetOptionBool('Daemons', 'accounts_daemon'):
    AccountsDaemon(
        groups=instance_config.GetOptionString('Accounts', 'groups'),
        remove=instance_config.GetOptionBool('Accounts', 'deprovision_remove'),
        useradd_cmd=instance_config.GetOptionString('Accounts', 'useradd_cmd'),
        userdel_cmd=instance_config.GetOptionString('Accounts', 'userdel_cmd'),
        usermod_cmd=instance_config.GetOptionString('Accounts', 'usermod_cmd'),
        groupadd_cmd=instance_config.GetOptionString(
            'Accounts', 'groupadd_cmd'),
        gpasswd_add_cmd=instance_config.GetOptionString('Accounts', 'gpasswd_add_cmd'),
        gpasswd_remove_cmd=instance_config.GetOptionString('Accounts', 'gpasswd_remove_cmd'),
        debug=bool(options.debug))


if __name__ == '__main__':
  main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for file_utils.py module."""

from google_compute_engine import file_utils
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class FileUtilsTest(unittest.TestCase):

  def setUp(self):
    self.fd = 1
    self.path = '/tmp/path'

  @mock.patch('google_compute_engine.file_utils.subprocess.call')
  @mock.patch('google_compute_engine.file_utils.os.access')
  @mock.patch('google_compute_engine.file_utils.os.path.isfile')
  def testSetSELinuxContext(self, mock_isfile, mock_access, mock_call):
    restorecon = '/sbin/restorecon'
    path = 'path'
    mock_isfile.return_value = True
    mock_access.return_value = True
    file_utils._SetSELinuxContext(path)
    mock_isfile.assert_called_once_with(restorecon)
    mock_access.assert_called_once_with(restorecon, file_utils.os.X_OK)
    mock_call.assert_called_once_with([restorecon, path])

  @mock.patch('google_compute_engine.file_utils.subprocess.call')
  @mock.patch('google_compute_engine.file_utils.os.access')
  @mock.patch('google_compute_engine.file_utils.os.path.isfile')
  def testSetSELinuxContextSkip(self, mock_isfile, mock_access, mock_call):
    mock_isfile.side_effect = [True, False, False]
    mock_access.side_effect = [False, True, False]
    file_utils._SetSELinuxContext('1')
    file_utils._SetSELinuxContext('2')
    file_utils._SetSELinuxContext('3')
    mock_call.assert_not_called()

  @mock.patch('google_compute_engine.file_utils._SetSELinuxContext')
  @mock.patch('google_compute_engine.file_utils.os.path.exists')
  @mock.patch('google_compute_engine.file_utils.os.mkdir')
  @mock.patch('google_compute_engine.file_utils.os.chown')
  @mock.patch('google_compute_engine.file_utils.os.chmod')
  def testSetPermissions(
      self, mock_chmod, mock_chown, mock_mkdir, mock_exists, mock_context):
    mocks = mock.Mock()
    mocks.attach_mock(mock_chmod, 'chmod')
    mocks.attach_mock(mock_chown, 'chown')
    mocks.attach_mock(mock_mkdir, 'mkdir')
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_context, 'context')
    path = 'path'
    mode = 'mode'
    uid = 'uid'
    gid = 'gid'
    mock_exists.side_effect = [False, True, False]

    # Create a new directory.
    file_utils.SetPermissions(path, mode=mode, uid=uid, gid=gid, mkdir=True)
    # The path exists, so do not create a new directory.
    file_utils.SetPermissions(path, mode=mode, uid=uid, gid=gid, mkdir=True)
    # Create a new directory without a mode specified.
    file_utils.SetPermissions(path, uid=uid, gid=gid, mkdir=True)
    # Do not create the path even though it does not exist.
    file_utils.SetPermissions(path, mode=mode, uid=uid, gid=gid, mkdir=False)
    # Do not set an owner when a UID or GID is not specified.
    file_utils.SetPermissions(path, mode=mode, mkdir=False)
    # Set the SELinux context when no parameters are specified.
    file_utils.SetPermissions(path)
    expected_calls = [
        # Create a new directory.
        mock.call.exists(path),
        mock.call.mkdir(path, mode),
        mock.call.chown(path, uid, gid),
        mock.call.context(path),
        # Attempt to create a new path but reuse existing path.
        mock.call.exists(path),
        mock.call.chmod(path, mode),
        mock.call.chown(path, uid, gid),
        mock.call.context(path),
        # Create a new directory with default mode.
        mock.call.exists(path),
        mock.call.mkdir(path, 0o777),
        mock.call.chown(path, uid, gid),
        mock.call.context(path),
        # Set permissions and owner on an existing path.
        mock.call.chmod(path, mode),
        mock.call.chown(path, uid, gid),
        mock.call.context(path),
        # Set permissions, without changing ownership, of an existing path.
        mock.call.chmod(path, mode),
        mock.call.context(path),
        # Set SELinux context on an existing path.
        mock.call.context(path),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testLock(self, mock_flock):
    operation = file_utils.fcntl.LOCK_EX | file_utils.fcntl.LOCK_NB
    file_utils.Lock(self.fd, self.path, False)
    mock_flock.assert_called_once_with(self.fd, operation)

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testLockBlocking(self, mock_flock):
    operation = file_utils.fcntl.LOCK_EX
    file_utils.Lock(self.fd, self.path, True)
    mock_flock.assert_called_once_with(self.fd, operation)

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testLockTakenException(self, mock_flock):
    error = IOError('Test Error')
    error.errno = file_utils.errno.EWOULDBLOCK
    mock_flock.side_effect = error
    try:
      file_utils.Lock(self.fd, self.path, False)
    except IOError as e:
      self.assertTrue(self.path in str(e))

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testLockException(self, mock_flock):
    error = IOError('Test Error')
    mock_flock.side_effect = error
    try:
      file_utils.Lock(self.fd, self.path, False)
    except IOError as e:
      self.assertTrue(self.path in str(e))
      self.assertTrue('Test Error' in str(e))

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testUnlock(self, mock_flock):
    operation = file_utils.fcntl.LOCK_UN | file_utils.fcntl.LOCK_NB
    file_utils.Unlock(self.fd, self.path)
    mock_flock.assert_called_once_with(self.fd, operation)

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testUnlockTakenException(self, mock_flock):
    error = IOError('Test Error')
    error.errno = file_utils.errno.EWOULDBLOCK
    mock_flock.side_effect = error
    try:
      file_utils.Unlock(self.fd, self.path)
    except IOError as e:
      self.assertTrue(self.path in str(e))

  @mock.patch('google_compute_engine.file_utils.fcntl.flock')
  def testUnlockException(self, mock_flock):
    error = IOError('Test Error')
    mock_flock.side_effect = error
    try:
      file_utils.Unlock(self.fd, self.path)
    except IOError as e:
      self.assertTrue(self.path in str(e))
      self.assertTrue('Test Error' in str(e))

  @mock.patch('google_compute_engine.file_utils.Unlock')
  @mock.patch('google_compute_engine.file_utils.Lock')
  @mock.patch('google_compute_engine.file_utils.os')
  def testLockFile(self, mock_os, mock_lock, mock_unlock):
    mock_callable = mock.Mock()
    mock_os.open.return_value = self.fd
    mock_os.O_CREAT = 1
    mocks = mock.Mock()
    mocks.attach_mock(mock_callable, 'callable')
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_unlock, 'unlock')
    mocks.attach_mock(mock_os.open, 'open')
    mocks.attach_mock(mock_os.close, 'close')

    with file_utils.LockFile(self.path, blocking=True):
      mock_callable('test')

    expected_calls = [
        mock.call.open(self.path, 1),
        mock.call.lock(self.fd, self.path, True),
        mock.call.callable('test'),
        mock.call.unlock(self.fd, self.path),
        mock.call.close(1),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library for watching changes in the metadata server."""

import functools
import json
import logging
import os
import socket
import time

from google_compute_engine.compat import httpclient
from google_compute_engine.compat import urlerror
from google_compute_engine.compat import urlparse
from google_compute_engine.compat import urlrequest

METADATA_SERVER = 'http://metadata.google.internal/computeMetadata/v1'


class StatusException(urlerror.HTTPError):

  def __init__(self, response):
    url = response.geturl()
    code = response.getcode()
    message = httpclient.responses.get(code)
    headers = response.headers
    super(StatusException, self).__init__(url, code, message, headers, response)


def RetryOnUnavailable(func):
  """Function decorator to retry on a service unavailable exception."""

  @functools.wraps(func)
  def Wrapper(*args, **kwargs):
    while True:
      try:
        response = func(*args, **kwargs)
      except (httpclient.HTTPException, socket.error, urlerror.URLError) as e:
        time.sleep(5)
        if (isinstance(e, urlerror.HTTPError)
            and e.getcode() == httpclient.SERVICE_UNAVAILABLE):
          continue
        elif isinstance(e, socket.timeout):
          continue
        raise
      else:
        if response.getcode() == httpclient.OK:
          return response
        else:
          raise StatusException(response)
  return Wrapper


class MetadataWatcher(object):
  """Watches for changes in metadata."""

  def __init__(self, logger=None, timeout=60):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      timeout: int, timeout in seconds for metadata requests.
    """
    self.etag = 0
    self.logger = logger or logging
    self.timeout = timeout

  @RetryOnUnavailable
  def _GetMetadataRequest(self, metadata_url, params=None, timeout=None):
    """Performs a GET request with the metadata headers.

    Args:
      metadata_url: string, the URL to perform a GET request on.
      params: dictionary, the query parameters in the GET request.
      timeout: int, timeout in seconds for metadata requests.

    Returns:
      HTTP response from the GET request.

    Raises:
      urlerror.HTTPError: raises when the GET request fails.
    """
    headers = {'Metadata-Flavor': 'Google'}
    params = urlparse.urlencode(params or {})
    url = '%s?%s' % (metadata_url, params)
    request = urlrequest.Request(url, headers=headers)
    request_opener = urlrequest.build_opener(urlrequest.ProxyHandler({}))
    timeout = timeout or self.timeout
    return request_opener.open(request, timeout=timeout*1.1)

  def _UpdateEtag(self, response):
    """Update the etag from an API response.

    Args:
      response: HTTP response with a header field.

    Returns:
      bool, True if the etag in the response header updated.
    """
    etag = response.headers.get('etag', self.etag)
    etag_updated = self.etag != etag
    self.etag = etag
    return etag_updated

  def _GetMetadataUpdate(
      self, metadata_key='', recursive=True, wait=True, timeout=None):
    """Request the contents of metadata server and deserialize the response.

    Args:
      metadata_key: string, the metadata key to watch for changes.
      recursive: bool, True if we should recursively watch for metadata changes.
      wait: bool, True if we should wait for a metadata change.
      timeout: int, timeout in seconds for returning metadata output.

    Returns:
      json, the deserialized contents of the metadata server.
    """
    metadata_key = os.path.join(metadata_key, '') if recursive else metadata_key
    metadata_url = os.path.join(METADATA_SERVER, metadata_key)
    params = {
        'alt': 'json',
        'last_etag': self.etag,
        'recursive': recursive,
        'timeout_sec': timeout or self.timeout,
        'wait_for_change': wait,
    }
    while True:
      response = self._GetMetadataRequest(
          metadata_url, params=params, timeout=timeout)
      etag_updated = self._UpdateEtag(response)
      if wait and not etag_updated and not timeout:
        # Retry until the etag is updated.
        continue
      else:
        # One of the following are true:
        # - Waiting for change is not required.
        # - The etag is updated.
        # - The user specified a request timeout.
        break
    return json.loads(response.read().decode('utf-8'))

  def _HandleMetadataUpdate(
      self, metadata_key='', recursive=True, wait=True, timeout=None,
      retry=True):
    """Wait for a successful metadata response.

    Args:
      metadata_key: string, the metadata key to watch for changes.
      recursive: bool, True if we should recursively watch for metadata changes.
      wait: bool, True if we should wait for a metadata change.
      timeout: int, timeout in seconds for returning metadata output.
      retry: bool, True if we should retry on failure.

    Returns:
      json, the deserialized contents of the metadata server.
    """
    exception = None
    while True:
      try:
        return self._GetMetadataUpdate(
            metadata_key=metadata_key, recursive=recursive, wait=wait,
            timeout=timeout)
      except (httpclient.HTTPException, socket.error, urlerror.URLError) as e:
        if not isinstance(e, type(exception)):
          exception = e
          self.logger.error('GET request error retrieving metadata. %s.', e)
        if retry:
          continue
        else:
          break

  def WatchMetadata(
      self, handler, metadata_key='', recursive=True, timeout=None):
    """Watch for changes to the contents of the metadata server.

    Args:
      handler: callable, a function to call with the updated metadata contents.
      metadata_key: string, the metadata key to watch for changes.
      recursive: bool, True if we should recursively watch for metadata changes.
      timeout: int, timeout in seconds for returning metadata output.
    """
    while True:
      response = self._HandleMetadataUpdate(
          metadata_key=metadata_key, recursive=recursive, wait=True,
          timeout=timeout)
      try:
        handler(response)
      except Exception as e:
        self.logger.exception('Exception calling the response handler. %s.', e)

  def GetMetadata(
      self, metadata_key='', recursive=True, timeout=None, retry=True):
    """Retrieve the contents of metadata server for a metadata key.

    Args:
      metadata_key: string, the metadata key to watch for changes.
      recursive: bool, True if we should recursively watch for metadata changes.
      timeout: int, timeout in seconds for returning metadata output.
      retry: bool, True if we should retry on failure.

    Returns:
      json, the deserialized contents of the metadata server or None if error.
    """
    return self._HandleMetadataUpdate(
        metadata_key=metadata_key, recursive=recursive, wait=False,
        timeout=timeout, retry=retry)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Execute user provided metadata scripts."""

import os
import stat
import subprocess


class ScriptExecutor(object):
  """A class for executing user provided metadata scripts."""

  def __init__(self, logger, script_type, default_shell=None):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      script_type: string, the type of the script we are running.
      default_shell: string, the default shell to execute the script.
    """
    self.logger = logger
    self.script_type = script_type
    self.default_shell = default_shell or '/bin/bash'

  def _MakeExecutable(self, metadata_script):
    """Add executable permissions to a file.

    Args:
      metadata_script: string, the path to the executable file.
    """
    mode = os.stat(metadata_script).st_mode
    os.chmod(metadata_script, mode | stat.S_IEXEC)

  def _RunScript(self, metadata_key, metadata_script):
    """Run a script and log the streamed script output.

    Args:
      metadata_key: string, the key specifing the metadata script.
      metadata_script: string, the file location of an executable script.
    """
    process = subprocess.Popen(
        metadata_script, shell=True,
        executable=self.default_shell,
        stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
    while True:
      for line in iter(process.stdout.readline, b''):
        message = line.decode('utf-8', 'replace').rstrip('\n')
        if message:
          self.logger.info('%s: %s', metadata_key, message)
      if process.poll() is not None:
        break
    self.logger.info('%s: Return code %s.', metadata_key, process.returncode)

  def RunScripts(self, script_dict):
    """Run the metadata scripts; execute a URL script first if one is provided.

    Args:
      script_dict: a dictionary mapping metadata keys to script files.
    """
    metadata_types = ['%s-script-url', '%s-script']
    metadata_keys = [key % self.script_type for key in metadata_types]
    metadata_keys = [key for key in metadata_keys if script_dict.get(key)]
    if not metadata_keys:
      self.logger.info('No %s scripts found in metadata.', self.script_type)
    for metadata_key in metadata_keys:
      metadata_script = script_dict.get(metadata_key)
      self._MakeExecutable(metadata_script)
      self._RunScript(metadata_key, metadata_script)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for script_retriever.py module."""

import subprocess

from google_compute_engine.compat import urlerror
from google_compute_engine.metadata_scripts import script_retriever
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class ScriptRetrieverTest(unittest.TestCase):

  def setUp(self):
    self.script_type = 'test'
    self.dest_dir = '/tmp'
    self.dest = '/tmp/file'
    self.mock_logger = mock.Mock()
    self.mock_watcher = mock.Mock()
    self.retriever = script_retriever.ScriptRetriever(
        self.mock_logger, self.script_type)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlrequest.Request')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlrequest.urlopen')
  def testDownloadAuthUrl(self, mock_urlopen, mock_request, mock_tempfile):
    auth_url = 'https://storage.googleapis.com/fake/url'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    self.retriever.token = 'bar'

    mock_open = mock.mock_open()
    with mock.patch('%s.open' % builtin, mock_open):
      self.assertEqual(
          self.retriever._DownloadAuthUrl(auth_url, self.dest_dir), self.dest)

    mock_tempfile.assert_called_once_with(dir=self.dest_dir, delete=False)
    mock_tempfile.close.assert_called_once_with()

    self.mock_logger.info.assert_called_once_with(
        mock.ANY, auth_url, self.dest)
    mock_request.assert_called_with(auth_url)
    mocked_request = mock_request()
    mocked_request.add_unredirected_header.assert_called_with(
        'Authorization', 'bar')
    mock_urlopen.assert_called_with(mocked_request)
    urlopen_read = mock_urlopen().read(return_value=b'foo')
    self.mock_logger.warning.assert_not_called()

    mock_open.assert_called_once_with(self.dest, 'wb')
    handle = mock_open()
    handle.write.assert_called_once_with(urlopen_read)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlrequest.Request')
  @mock.patch('google_compute_engine.metadata_watcher.MetadataWatcher.GetMetadata')
  def testDownloadAuthUrlExceptionAndToken(
      self, mock_get_metadata, mock_request, mock_tempfile):
    auth_url = 'https://storage.googleapis.com/fake/url'
    metadata_prefix = 'http://metadata.google.internal/computeMetadata/v1/'
    token_url = metadata_prefix + 'instance/service-accounts/default/token'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    self.retriever.token = None

    mock_get_metadata.return_value = {
        'token_type': 'foo', 'access_token': 'bar'}
    mock_request.return_value = mock_request
    mock_request.side_effect = urlerror.URLError('Error.')

    self.assertIsNone(self.retriever._DownloadAuthUrl(auth_url, self.dest_dir))

    mock_get_metadata.return_value = mock_get_metadata
    # GetMetadata includes a prefix, so remove it.
    stripped_url = token_url.replace(metadata_prefix, '')
    mock_get_metadata.assert_called_once_with(
        stripped_url, recursive=False, retry=False)

    self.assertEqual(self.retriever.token, 'foo bar')

    self.mock_logger.info.assert_called_once_with(
        mock.ANY, auth_url, self.dest)
    self.assertEqual(self.mock_logger.warning.call_count, 1)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.ScriptRetriever._DownloadUrl')
  @mock.patch('google_compute_engine.metadata_watcher.MetadataWatcher.GetMetadata')
  def testDownloadAuthUrlFallback(
      self, mock_get_metadata, mock_download_url, mock_tempfile):
    auth_url = 'https://storage.googleapis.com/fake/url'
    metadata_prefix = 'http://metadata.google.internal/computeMetadata/v1/'
    token_url = metadata_prefix + 'instance/service-accounts/default/token'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    self.retriever.token = None

    mock_get_metadata.return_value = None
    mock_download_url.return_value = None

    self.assertIsNone(self.retriever._DownloadAuthUrl(auth_url, self.dest_dir))

    mock_get_metadata.return_value = mock_get_metadata
    # GetMetadata includes a prefix, so remove it.
    prefix = 'http://metadata.google.internal/computeMetadata/v1/'
    stripped_url = token_url.replace(prefix, '')
    mock_get_metadata.assert_called_once_with(
        stripped_url, recursive=False, retry=False)
    mock_download_url.assert_called_once_with(auth_url, self.dest_dir)

    self.assertIsNone(self.retriever.token)

    expected_calls = [
        mock.call(mock.ANY, auth_url, self.dest),
        mock.call(mock.ANY),
    ]
    self.assertEqual(self.mock_logger.info.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlretrieve.urlretrieve')
  def testDownloadUrl(self, mock_retrieve, mock_tempfile):
    url = 'http://www.google.com/fake/url'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    self.assertEqual(
        self.retriever._DownloadUrl(url, self.dest_dir), self.dest)
    mock_tempfile.assert_called_once_with(dir=self.dest_dir, delete=False)
    mock_tempfile.close.assert_called_once_with()
    self.mock_logger.info.assert_called_once_with(mock.ANY, url, self.dest)
    mock_retrieve.assert_called_once_with(url, self.dest)
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.time')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlretrieve.urlretrieve')
  def testDownloadUrlProcessError(self, mock_retrieve, mock_tempfile, mock_time):
    url = 'http://www.google.com/fake/url'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    mock_success = mock.Mock()
    mock_success.getcode.return_value = script_retriever.httpclient.OK
    # Success after 3 timeout. Since max_retry = 3, the final result is fail.
    mock_retrieve.side_effect = [
        script_retriever.socket.timeout(),
        script_retriever.socket.timeout(),
        script_retriever.socket.timeout(),
        mock_success,
    ]
    self.assertIsNone(self.retriever._DownloadUrl(url, self.dest_dir))
    self.assertEqual(self.mock_logger.warning.call_count, 1)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.time')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlretrieve.urlretrieve')
  def testDownloadUrlWithRetry(self, mock_retrieve, mock_tempfile, mock_time):
    url = 'http://www.google.com/fake/url'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    mock_success = mock.Mock()
    mock_success.getcode.return_value = script_retriever.httpclient.OK
    # Success after 2 timeout. Since max_retry = 3, the final result is success.
    mock_retrieve.side_effect = [
        script_retriever.socket.timeout(),
        script_retriever.socket.timeout(),
        mock_success,
    ]
    self.assertIsNotNone(self.retriever._DownloadUrl(url, self.dest_dir))

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.urlretrieve.urlretrieve')
  def testDownloadUrlException(self, mock_retrieve, mock_tempfile):
    url = 'http://www.google.com/fake/url'
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.name = self.dest
    mock_retrieve.side_effect = Exception('Error.')
    self.assertIsNone(self.retriever._DownloadUrl(url, self.dest_dir))
    self.assertEqual(self.mock_logger.warning.call_count, 1)

  def _CreateUrls(self, bucket, obj, gs_match=True):
    """Creates a URL for each of the supported Google Storage URL formats.

    Args:
      bucket: string, the Google Storage bucket name.
      obj: string, the object name in the bucket.
      gs_match: bool, True if the bucket and object names are valid.

    Returns:
      (list, dict):
      list, the URLs to download.
      dict, a Google Storage URL mapped to the expected 'gs://' format.
    """
    gs_url = 'gs://%s/%s' % (bucket, obj)
    gs_urls = {gs_url: gs_url}
    url_formats = [
        'http://%s.storage.googleapis.com/%s',
        'https://%s.storage.googleapis.com/%s',
        'http://storage.googleapis.com/%s/%s',
        'https://storage.googleapis.com/%s/%s',
        'http://commondatastorage.googleapis.com/%s/%s',
        'https://commondatastorage.googleapis.com/%s/%s',
    ]
    url_formats = [url % (bucket, obj) for url in url_formats]
    if gs_match:
      gs_urls.update(dict((url, gs_url) for url in url_formats))
      return ([], gs_urls)
    else:
      return (url_formats, gs_urls)

  def testDownloadScript(self):
    mock_auth_download = mock.Mock()
    self.retriever._DownloadAuthUrl = mock_auth_download
    mock_download = mock.Mock()
    self.retriever._DownloadUrl = mock_download
    download_urls = []
    download_gs_urls = {}

    component_urls = [
        ('@#$%^', '\n\n\n\n', False),
        ('///////', '///////', False),
        ('Abc', 'xyz', False),
        (' abc', 'xyz', False),
        ('abc', 'xyz?', False),
        ('abc', 'xyz*', False),
        ('', 'xyz', False),
        ('a', 'xyz', False),
        ('abc', '', False),
        ('hello', 'world', True),
        ('hello', 'world!', True),
        ('hello', 'world !', True),
        ('hello', 'w o r l d ', True),
        ('hello', 'w\no\nr\nl\nd ', True),
        ('123_hello', '1!@#$%^', True),
        ('123456', 'hello.world', True),
    ]

    for bucket, obj, gs_match in component_urls:
      urls, gs_urls = self._CreateUrls(bucket, obj, gs_match=gs_match)
      download_urls.extend(urls)
      download_gs_urls.update(gs_urls)

    # All Google Storage URLs are downloaded with an authentication token.
    for url, gs_url in download_gs_urls.items():
      mock_download.reset_mock()
      mock_auth_download.reset_mock()
      self.retriever._DownloadScript(gs_url, self.dest_dir)
      new_gs_url = gs_url.replace('gs://', 'https://storage.googleapis.com/')
      mock_auth_download.assert_called_once_with(new_gs_url, self.dest_dir)
      mock_download.assert_not_called()

    for url in download_urls:
      mock_download.reset_mock()
      self.retriever._DownloadScript(url, self.dest_dir)
      mock_download.assert_called_once_with(url, self.dest_dir)

    for url, gs_url in download_gs_urls.items():
      if url.startswith('gs://'):
        continue
      mock_auth_download.reset_mock()
      mock_auth_download.return_value = None
      mock_download.reset_mock()
      self.retriever._DownloadScript(url, self.dest_dir)
      mock_auth_download.assert_called_once_with(url, self.dest_dir)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  def testGetAttributeScripts(self, mock_tempfile):
    script = 'echo Hello World.\n'
    script_dest = '/tmp/script'
    script_url = 'gs://fake/url'
    script_url_dest = '/tmp/script_url'
    attribute_data = {
        '%s-script' % self.script_type: '\n%s' % script,
        '%s-script-url' % self.script_type: script_url,
    }
    expected_data = {
        '%s-script' % self.script_type: script_dest,
        '%s-script-url' % self.script_type: script_url_dest,
    }
    # Mock saving a script to a file.
    mock_dest = mock.Mock()
    mock_dest.name = script_dest
    mock_tempfile.__enter__.return_value = mock_dest
    mock_tempfile.return_value = mock_tempfile
    # Mock downloading a script from a URL.
    mock_download = mock.Mock()
    mock_download.return_value = script_url_dest
    self.retriever._DownloadScript = mock_download

    self.assertEqual(
        self.retriever._GetAttributeScripts(attribute_data, self.dest_dir),
        expected_data)
    self.assertEqual(self.mock_logger.info.call_count, 2)
    mock_dest.write.assert_called_once_with(script)
    mock_download.assert_called_once_with(script_url, self.dest_dir)

  def testGetAttributeScriptsNone(self):
    attribute_data = {}
    expected_data = {}
    self.assertEqual(
        self.retriever._GetAttributeScripts(attribute_data, self.dest_dir),
        expected_data)
    self.mock_logger.info.assert_not_called()

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  def testGetScripts(self, mock_tempfile):
    script_dest = '/tmp/script'
    script_url_dest = '/tmp/script_url'
    metadata = {
        'instance': {
            'attributes': {
                '%s-script' % self.script_type: 'a',
                '%s-script-url' % self.script_type: 'b',
            },
        },
        'project': {
            'attributes': {
                '%s-script' % self.script_type: 'c',
                '%s-script-url' % self.script_type: 'd',
            },
        },
    }
    expected_data = {
        '%s-script' % self.script_type: script_dest,
        '%s-script-url' % self.script_type: script_url_dest,
    }
    self.mock_watcher.GetMetadata.return_value = metadata
    self.retriever.watcher = self.mock_watcher
    # Mock saving a script to a file.
    mock_dest = mock.Mock()
    mock_dest.name = script_dest
    mock_tempfile.__enter__.return_value = mock_dest
    mock_tempfile.return_value = mock_tempfile
    # Mock downloading a script from a URL.
    mock_download = mock.Mock()
    mock_download.return_value = script_url_dest
    self.retriever._DownloadScript = mock_download

    self.assertEqual(self.retriever.GetScripts(self.dest_dir), expected_data)
    self.assertEqual(self.mock_logger.info.call_count, 2)
    self.assertEqual(self.mock_logger.warning.call_count, 0)
    mock_dest.write.assert_called_once_with('a')
    mock_download.assert_called_once_with('b', self.dest_dir)

  def testGetScriptsNone(self):
    metadata = {
        'instance': {
            'attributes': None,
        },
        'project': {
            'attributes': None,
        },
    }
    expected_data = {}
    self.mock_watcher.GetMetadata.return_value = metadata
    self.retriever.watcher = self.mock_watcher
    self.assertEqual(self.retriever.GetScripts(self.dest_dir), expected_data)
    self.mock_logger.info.assert_not_called()

  def testGetScriptsNoMetadata(self):
    metadata = None
    expected_data = {}
    self.mock_watcher.GetMetadata.return_value = metadata
    self.retriever.watcher = self.mock_watcher
    self.assertEqual(self.retriever.GetScripts(self.dest_dir), expected_data)
    self.mock_logger.info.assert_not_called()
    self.assertEqual(self.mock_logger.warning.call_count, 2)

  @mock.patch('google_compute_engine.metadata_scripts.script_retriever.tempfile.NamedTemporaryFile')
  def testGetScriptsFailed(self, mock_tempfile):
    script_dest = '/tmp/script'
    script_url_dest = None
    metadata = {
        'instance': {
            'attributes': {
                '%s-script' % self.script_type: 'a',
                '%s-script-url' % self.script_type: 'b',
            },
        },
        'project': {
            'attributes': {
                '%s-script' % self.script_type: 'c',
                '%s-script-url' % self.script_type: 'd',
            },
        },
    }
    expected_data = {
        '%s-script' % self.script_type: script_dest,
        '%s-script-url' % self.script_type: script_url_dest,
    }
    self.mock_watcher.GetMetadata.return_value = metadata
    self.retriever.watcher = self.mock_watcher
    # Mock saving a script to a file.
    mock_dest = mock.Mock()
    mock_dest.name = script_dest
    mock_tempfile.__enter__.return_value = mock_dest
    mock_tempfile.return_value = mock_tempfile
    # Mock downloading a script from a URL.
    mock_download = mock.Mock()
    mock_download.return_value = None
    self.retriever._DownloadScript = mock_download

    self.assertEqual(self.retriever.GetScripts(self.dest_dir), expected_data)
    self.assertEqual(self.mock_logger.info.call_count, 2)
    self.assertEqual(self.mock_logger.warning.call_count, 1)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for metadata_watcher.py module."""

import os

from google_compute_engine import metadata_watcher
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class MetadataWatcherTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.timeout = 60
    self.url = 'http://metadata.google.internal/computeMetadata/v1'
    self.params = {
        'alt': 'json',
        'last_etag': 0,
        'recursive': True,
        'timeout_sec': self.timeout,
        'wait_for_change': True,
    }
    self.mock_watcher = metadata_watcher.MetadataWatcher(
        logger=self.mock_logger, timeout=self.timeout)

  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.build_opener')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.ProxyHandler')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.Request')
  def testGetMetadataRequest(self, mock_request, mock_proxy, mock_opener):
    mock_open = mock.Mock()
    mock_handler = mock.Mock()
    mock_response = mock.Mock()
    mocks = mock.Mock()
    mocks.attach_mock(mock_request, 'request')
    mocks.attach_mock(mock_proxy, 'proxy')
    mocks.attach_mock(mock_handler, 'handler')
    mocks.attach_mock(mock_opener, 'opener')
    mocks.attach_mock(mock_open, 'open')
    mocks.attach_mock(mock_response, 'response')
    mock_request.return_value = mock_request
    mock_proxy.return_value = mock_handler
    mock_opener.return_value = mock_open
    mock_response.getcode.return_value = metadata_watcher.httpclient.OK
    mock_open.open.return_value = mock_response
    params = {'hello': 'world'}
    request_url = '%s?hello=world' % self.url
    headers = {'Metadata-Flavor': 'Google'}
    timeout = self.timeout * 1.1

    self.mock_watcher._GetMetadataRequest(self.url, params=params)
    expected_calls = [
        mock.call.request(request_url, headers=headers),
        mock.call.proxy({}),
        mock.call.opener(mock_handler),
        mock.call.open.open(mock_request, timeout=timeout),
        mock.call.response.getcode(),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.metadata_watcher.time')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.build_opener')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.ProxyHandler')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.Request')
  def testGetMetadataRequestRetry(
      self, mock_request, mock_proxy, mock_opener, mock_time):
    mock_open = mock.Mock()
    mock_handler = mock.Mock()
    mocks = mock.Mock()
    mocks.attach_mock(mock_request, 'request')
    mocks.attach_mock(mock_proxy, 'proxy')
    mocks.attach_mock(mock_handler, 'handler')
    mocks.attach_mock(mock_opener, 'opener')
    mocks.attach_mock(mock_open, 'open')
    mocks.attach_mock(mock_time, 'time')
    mock_request.return_value = mock_request
    mock_proxy.return_value = mock_handler
    mock_opener.return_value = mock_open

    mock_unavailable = mock.Mock()
    mock_unavailable.getcode.return_value = (
        metadata_watcher.httpclient.SERVICE_UNAVAILABLE)
    mock_timeout = metadata_watcher.socket.timeout('Test timeout')
    mock_success = mock.Mock()
    mock_success.getcode.return_value = metadata_watcher.httpclient.OK

    # Retry after a service unavailable error response.
    mock_open.open.side_effect = [
        metadata_watcher.StatusException(mock_unavailable),
        mock_timeout,
        mock_success,
    ]
    request_url = '%s?' % self.url
    headers = {'Metadata-Flavor': 'Google'}
    timeout = self.timeout * 1.1

    self.mock_watcher._GetMetadataRequest(self.url)
    expected_calls = [
        mock.call.request(request_url, headers=headers),
        mock.call.proxy({}),
        mock.call.opener(mock_handler),
        mock.call.open.open(mock_request, timeout=timeout),
        mock.call.time.sleep(mock.ANY),
        mock.call.request(request_url, headers=headers),
        mock.call.proxy({}),
        mock.call.opener(mock_handler),
        mock.call.open.open(mock_request, timeout=timeout),
        mock.call.time.sleep(mock.ANY),
        mock.call.request(request_url, headers=headers),
        mock.call.proxy({}),
        mock.call.opener(mock_handler),
        mock.call.open.open(mock_request, timeout=timeout),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.metadata_watcher.time')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.build_opener')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.ProxyHandler')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.Request')
  def testGetMetadataRequestHttpException(
      self, mock_request, mock_proxy, mock_opener, mock_time):
    mock_open = mock.Mock()
    mock_handler = mock.Mock()
    mock_response = mock.Mock()
    mock_request.return_value = mock_request
    mock_proxy.return_value = mock_handler
    mock_opener.return_value = mock_open
    mock_response.getcode.return_value = metadata_watcher.httpclient.NOT_FOUND
    mock_open.open.side_effect = metadata_watcher.StatusException(mock_response)

    with self.assertRaises(metadata_watcher.StatusException):
      self.mock_watcher._GetMetadataRequest(self.url)
    self.assertEqual(mock_request.call_count, 1)
    self.assertEqual(mock_proxy.call_count, 1)
    self.assertEqual(mock_opener.call_count, 1)
    self.assertEqual(mock_open.open.call_count, 1)
    self.assertEqual(mock_response.getcode.call_count, 1)

  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.build_opener')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.ProxyHandler')
  @mock.patch('google_compute_engine.metadata_watcher.urlrequest.Request')
  def testGetMetadataRequestException(
      self, mock_request, mock_proxy, mock_opener):
    mock_open = mock.Mock()
    mock_handler = mock.Mock()
    mock_response = mock.Mock()
    mock_request.return_value = mock_request
    mock_proxy.return_value = mock_handler
    mock_opener.return_value = mock_open
    mock_response.getcode.return_value = metadata_watcher.httpclient.NOT_FOUND
    mock_open.open.side_effect = mock_response

    with self.assertRaises(metadata_watcher.StatusException):
      self.mock_watcher._GetMetadataRequest(self.url)
    self.assertEqual(mock_request.call_count, 1)
    self.assertEqual(mock_proxy.call_count, 1)
    self.assertEqual(mock_opener.call_count, 1)
    self.assertEqual(mock_open.open.call_count, 1)

  def testUpdateEtag(self):
    mock_response = mock.Mock()
    mock_response.headers = {'etag': 1}
    self.assertEqual(self.mock_watcher.etag, 0)

    # Update the etag if the etag is set.
    self.assertTrue(self.mock_watcher._UpdateEtag(mock_response))
    self.assertEqual(self.mock_watcher.etag, 1)

    # Do not update the etag if the etag is unchanged.
    self.assertFalse(self.mock_watcher._UpdateEtag(mock_response))
    self.assertEqual(self.mock_watcher.etag, 1)

    # Do not update the etag if the etag is not set.
    mock_response.headers = {}
    self.assertFalse(self.mock_watcher._UpdateEtag(mock_response))
    self.assertEqual(self.mock_watcher.etag, 1)

  def testGetMetadataUpdate(self):
    mock_response = mock.Mock()
    mock_response.return_value = mock_response
    mock_response.headers = {'etag': 1}
    mock_response.read.return_value = bytes(b'{}')
    self.mock_watcher._GetMetadataRequest = mock_response
    request_url = os.path.join(self.url, '')

    self.assertEqual(self.mock_watcher._GetMetadataUpdate(), {})
    self.assertEqual(self.mock_watcher.etag, 1)
    mock_response.assert_called_once_with(
        request_url, params=self.params, timeout=None)

  def testGetMetadataUpdateArgs(self):
    mock_response = mock.Mock()
    mock_response.return_value = mock_response
    mock_response.headers = {'etag': 0}
    mock_response.read.return_value = bytes(b'{}')
    self.mock_watcher._GetMetadataRequest = mock_response
    metadata_key = 'instance/id'
    self.params['recursive'] = False
    self.params['wait_for_change'] = False
    request_url = os.path.join(self.url, metadata_key)

    self.mock_watcher._GetMetadataUpdate(
        metadata_key=metadata_key, recursive=False, wait=False, timeout=60)
    self.assertEqual(self.mock_watcher.etag, 0)
    mock_response.assert_called_once_with(
        request_url, params=self.params, timeout=60)

  def testGetMetadataUpdateWait(self):
    self.params['last_etag'] = 1
    self.mock_watcher.etag = 1
    mock_unchanged = mock.Mock()
    mock_unchanged.headers = {'etag': 1}
    mock_unchanged.read.return_value = bytes(b'{}')
    mock_changed = mock.Mock()
    mock_changed.headers = {'etag': 2}
    mock_changed.read.return_value = bytes(b'{}')
    mock_response = mock.Mock()
    mock_response.side_effect = [mock_unchanged, mock_unchanged, mock_changed]
    self.mock_watcher._GetMetadataRequest = mock_response
    request_url = os.path.join(self.url, '')

    self.mock_watcher._GetMetadataUpdate()
    self.assertEqual(self.mock_watcher.etag, 2)
    expected_calls = [
        mock.call(request_url, params=self.params, timeout=None),
    ] * 3
    self.assertEqual(mock_response.mock_calls, expected_calls)

  def testHandleMetadataUpdate(self):
    mock_response = mock.Mock()
    mock_response.return_value = {}
    self.mock_watcher._GetMetadataUpdate = mock_response

    self.assertEqual(self.mock_watcher.GetMetadata(), {})
    mock_response.assert_called_once_with(
        metadata_key='', recursive=True, wait=False, timeout=None)
    self.mock_watcher.logger.exception.assert_not_called()

  def testHandleMetadataUpdateException(self):
    mock_response = mock.Mock()
    first = metadata_watcher.socket.timeout()
    second = metadata_watcher.urlerror.URLError('Test')
    mock_response.side_effect = [first, first, second, {}]
    self.mock_watcher._GetMetadataUpdate = mock_response
    metadata_key = 'instance/id'
    recursive = False
    wait = False
    retry = True

    self.assertEqual(
        self.mock_watcher._HandleMetadataUpdate(
            metadata_key=metadata_key, recursive=recursive, wait=wait,
            timeout=None, retry=retry),
        {})
    expected_calls = [
        mock.call(
          metadata_key=metadata_key, recursive=recursive, wait=wait,
          timeout=None),
    ] * 4
    self.assertEqual(mock_response.mock_calls, expected_calls)
    expected_calls = [mock.call.error(mock.ANY, mock.ANY)] * 2
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testHandleMetadataUpdateExceptionNoRetry(self):
    mock_response = mock.Mock()
    mock_response.side_effect = metadata_watcher.socket.timeout()
    self.mock_watcher._GetMetadataUpdate = mock_response
    metadata_key = 'instance/id'
    recursive = False
    wait = False
    retry = False

    self.assertIsNone(
        self.mock_watcher._HandleMetadataUpdate(
            metadata_key=metadata_key, recursive=recursive, wait=wait,
            timeout=None, retry=retry))
    expected_calls = [
        mock.call(
            metadata_key=metadata_key, recursive=recursive, wait=wait,
            timeout=None),
    ]
    self.assertEqual(mock_response.mock_calls, expected_calls)
    expected_calls = [mock.call.error(mock.ANY, mock.ANY)]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  def testWatchMetadata(self):
    mock_response = mock.Mock()
    mock_response.return_value = {}
    self.mock_watcher._HandleMetadataUpdate = mock_response
    mock_handler = mock.Mock()
    mock_handler.side_effect = Exception()
    self.mock_logger.exception.side_effect = RuntimeError()
    recursive = True

    with self.assertRaises(RuntimeError):
      self.mock_watcher.WatchMetadata(mock_handler, recursive=recursive)
    mock_handler.assert_called_once_with({})
    mock_response.assert_called_once_with(
        metadata_key='', recursive=recursive, wait=True, timeout=None)

  def testWatchMetadataException(self):
    mock_response = mock.Mock()
    mock_response.side_effect = metadata_watcher.socket.timeout()
    self.mock_watcher._GetMetadataUpdate = mock_response
    self.mock_logger.error.side_effect = RuntimeError()
    metadata_key = 'instance/id'
    recursive = False

    with self.assertRaises(RuntimeError):
      self.mock_watcher.WatchMetadata(
          None, metadata_key=metadata_key, recursive=recursive)
    mock_response.assert_called_once_with(
        metadata_key=metadata_key, recursive=recursive, wait=True, timeout=None)

  def testGetMetadata(self):
    mock_response = mock.Mock()
    mock_response.return_value = {}
    self.mock_watcher._HandleMetadataUpdate = mock_response

    self.assertEqual(self.mock_watcher.GetMetadata(), {})
    mock_response.assert_called_once_with(
        metadata_key='', recursive=True, wait=False, timeout=None, retry=True)
    self.mock_watcher.logger.exception.assert_not_called()

  def testGetMetadataArgs(self):
    mock_response = mock.Mock()
    mock_response.return_value = {}
    self.mock_watcher._HandleMetadataUpdate = mock_response
    metadata_key = 'instance/id'
    recursive = False
    retry = False

    response = self.mock_watcher.GetMetadata(
        metadata_key=metadata_key, recursive=recursive, timeout=60,
        retry=retry)
    self.assertEqual(response, {})
    mock_response.assert_called_once_with(
        metadata_key=metadata_key, recursive=False, wait=False, timeout=60,
        retry=False)
    self.mock_watcher.logger.exception.assert_not_called()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Manage networking on a Google Compute Engine instance.

Run network setup to enable multiple network interfaces on startup.
Update IP forwarding when metadata changes.
"""

import logging.handlers
import optparse
import random

from google_compute_engine import config_manager
from google_compute_engine import constants
from google_compute_engine import file_utils
from google_compute_engine import logger
from google_compute_engine import metadata_watcher
from google_compute_engine import network_utils
from google_compute_engine.networking.ip_forwarding import ip_forwarding
from google_compute_engine.networking.network_setup import network_setup

LOCKFILE = constants.LOCALSTATEDIR + '/lock/google_networking.lock'


class NetworkDaemon(object):
  """Manage networking based on changes to network metadata."""

  network_interface_metadata_key = 'instance/network-interfaces'

  def __init__(
      self, ip_forwarding_enabled, proto_id, ip_aliases, target_instance_ips,
      dhclient_script, dhcp_command, network_setup_enabled, debug=False):
    """Constructor.

    Args:
      ip_forwarding_enabled: bool, True if ip forwarding is enabled.
      proto_id: string, the routing protocol identifier for Google IP changes.
      ip_aliases: bool, True if the guest should configure IP alias routes.
      target_instance_ips: bool, True supports internal IP load balancing.
      dhclient_script: string, the path to a dhclient script used by dhclient.
      dhcp_command: string, a command to enable Ethernet interfaces.
      network_setup_enabled: bool, True if network setup is enabled.
      debug: bool, True if debug output should write to the console.
    """
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='google-networking', debug=debug, facility=facility)
    self.ip_aliases = ip_aliases
    self.ip_forwarding_enabled = ip_forwarding_enabled
    self.network_setup_enabled = network_setup_enabled
    self.target_instance_ips = target_instance_ips

    self.ip_forwarding = ip_forwarding.IpForwarding(
        proto_id=proto_id, debug=debug)
    self.network_setup = network_setup.NetworkSetup(
        dhclient_script=dhclient_script, dhcp_command=dhcp_command, debug=debug)
    self.network_utils = network_utils.NetworkUtils(logger=self.logger)
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)

    try:
      with file_utils.LockFile(LOCKFILE):
        self.logger.info('Starting Google Networking daemon.')
        timeout = 60 + random.randint(0, 30)
        self.watcher.WatchMetadata(
            self.HandleNetworkInterfaces,
            metadata_key=self.network_interface_metadata_key, recursive=True,
            timeout=timeout)
    except (IOError, OSError) as e:
      self.logger.warning(str(e))

  def HandleNetworkInterfaces(self, result):
    """Called when network interface metadata changes.

    Args:
      result: dict, the metadata response with the network interfaces.
    """
    network_interfaces = self._ExtractInterfaceMetadata(result)

    if self.network_setup_enabled:
      default_interface = network_interfaces[0]
      if default_interface.ipv6:
        self.network_setup.EnableIpv6([default_interface.name])
      else:
        self.network_setup.DisableIpv6([default_interface.name])
      self.network_setup.EnableNetworkInterfaces(
          [interface.name for interface in network_interfaces[1:]])

    for interface in network_interfaces:
      if self.ip_forwarding_enabled:
        self.ip_forwarding.HandleForwardedIps(
            interface.name, interface.forwarded_ips, interface.ip)

  def _ExtractInterfaceMetadata(self, metadata):
    """Extracts network interface metadata.

    Args:
      metadata: dict, the metadata response with the new network interfaces.

    Returns:
      list, a list of NetworkInterface objects.
    """
    interfaces = []
    for network_interface in metadata:
      mac_address = network_interface.get('mac')
      interface = self.network_utils.GetNetworkInterface(mac_address)
      ip_addresses = []
      if interface:
        ip_addresses.extend(network_interface.get('forwardedIps', []))
        if self.ip_aliases:
          ip_addresses.extend(network_interface.get('ipAliases', []))
        if self.target_instance_ips:
          ip_addresses.extend(network_interface.get('targetInstanceIps', []))
        interfaces.append(NetworkDaemon.NetworkInterface(
            interface, forwarded_ips=ip_addresses,
            ip=network_interface.get('ip', None),
            ipv6='dhcpv6Refresh' in network_interface.keys()))
      else:
        message = 'Network interface not found for MAC address: %s.'
        self.logger.warning(message, mac_address)
    return interfaces

  class NetworkInterface(object):
    """Network interface information extracted from metadata."""

    def __init__(self, name, forwarded_ips=None, ip=None, ipv6=False):
      self.name = name
      self.forwarded_ips = forwarded_ips
      self.ip = ip
      self.ipv6 = ipv6


def main():
  parser = optparse.OptionParser()
  parser.add_option(
      '-d', '--debug', action='store_true', dest='debug',
      help='print debug output to the console.')
  (options, _) = parser.parse_args()
  debug = bool(options.debug)
  instance_config = config_manager.ConfigManager()
  ip_forwarding_daemon_enabled = instance_config.GetOptionBool(
      'Daemons', 'ip_forwarding_daemon')
  ip_forwarding_enabled = instance_config.GetOptionBool(
      'NetworkInterfaces', 'ip_forwarding') or ip_forwarding_daemon_enabled
  network_setup_enabled = instance_config.GetOptionBool(
      'NetworkInterfaces', 'setup')
  network_daemon_enabled = instance_config.GetOptionBool(
      'Daemons', 'network_daemon')
  proto_id = instance_config.GetOptionString(
      'IpForwarding', 'ethernet_proto_id')
  ip_aliases = instance_config.GetOptionBool(
      'IpForwarding', 'ip_aliases')
  target_instance_ips = instance_config.GetOptionBool(
      'IpForwarding', 'target_instance_ips')
  dhclient_script = instance_config.GetOptionString(
      'NetworkInterfaces', 'dhclient_script')
  dhcp_command = instance_config.GetOptionString(
      'NetworkInterfaces', 'dhcp_command')

  if network_daemon_enabled:
    NetworkDaemon(
        ip_forwarding_enabled=ip_forwarding_enabled,
        proto_id=proto_id,
        ip_aliases=ip_aliases,
        target_instance_ips=target_instance_ips,
        dhclient_script=dhclient_script,
        dhcp_command=dhcp_command,
        network_setup_enabled=network_setup_enabled,
        debug=debug)


if __name__ == '__main__':
  main()



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Distro helpers."""

import os
import subprocess


def CallDhclient(
    interfaces, logger, dhclient_script=None):
  """Configure the network interfaces using dhclient.

  Args:
    interfaces: list of string, the output device names to enable.
    logger: logger object, used to write to SysLog and serial port.
    dhclient_script: string, the path to a dhclient script used by dhclient.
  """
  logger.info('Enabling the Ethernet interfaces %s.', interfaces)

  dhclient_command = ['dhclient']

  if dhclient_script and os.path.exists(dhclient_script):
    dhclient_command += ['-sf', dhclient_script]

  try:
    subprocess.check_call(dhclient_command + ['-x'] + interfaces)
    subprocess.check_call(dhclient_command + interfaces)
  except subprocess.CalledProcessError:
    logger.warning('Could not enable interfaces %s.', interfaces)


def CallDhclientIpv6(interfaces, logger, dhclient_script=None,
                     release_lease=False):
  """Configure the network interfaces for IPv6 using dhclient.

  Args:
    interface: string, the output device names for enabling IPv6.
    logger: logger object, used to write to SysLog and serial port.
    dhclient_script: string, the path to a dhclient script used by dhclient.
    release_lease: Release the IPv6 lease.
  """
  logger.info('Calling Dhclient for IPv6 configuration '
              'on the Ethernet interfaces %s.', interfaces)

  timeout_command = ['timeout', '5']
  dhclient_command = ['dhclient']

  if release_lease:
    try:
      subprocess.check_call(
          timeout_command + dhclient_command + [
              '-6', '-r', '-v'] + interfaces)
    except subprocess.CalledProcessError:
      logger.warning('Could not release IPv6 lease on interface %s.',
                     interfaces)
    return

  if dhclient_script and os.path.exists(dhclient_script):
    dhclient_command += ['-sf', dhclient_script]

  try:
    subprocess.check_call(
        timeout_command + dhclient_command + ['-1', '-6', '-v'] + interfaces)
  except subprocess.CalledProcessError:
    logger.warning('Could not enable IPv6 on interface %s.', interfaces)


def CallEnableRouteAdvertisements(interfaces, logger):
  """Enable route advertisements.
  Args:
    interfaces: list of string, the output device names to enable.
    logger: logger object, used to write to SysLog and serial port.
  """
  for interface in interfaces:
    accept_ra = (
        'net.ipv6.conf.{interface}.accept_ra_rt_info_max_plen'.format(
            interface=interface))
    CallSysctl(logger, accept_ra, 128)

def CallHwclock(logger):
  """Sync clock using hwclock.

  Args:
    logger: logger object, used to write to SysLog and serial port.
  """
  command = ['/sbin/hwclock', '--hctosys']
  try:
    subprocess.check_call(command)
  except subprocess.CalledProcessError:
    logger.warning('Failed to sync system time with hardware clock.')
  else:
    logger.info('Synced system time with hardware clock.')


def CallNtpdate(logger):
  """Sync clock using ntpdate.

  Args:
    logger: logger object, used to write to SysLog and serial port.
  """
  ntpd_inactive = subprocess.call(['service', 'ntpd', 'status'])
  try:
    if not ntpd_inactive:
      subprocess.check_call(['service', 'ntpd', 'stop'])
    subprocess.check_call(
        'ntpdate `awk \'$1=="server" {print $2}\' /etc/ntp.conf`', shell=True)
    if not ntpd_inactive:
      subprocess.check_call(['service', 'ntpd', 'start'])
  except subprocess.CalledProcessError:
    logger.warning('Failed to sync system time with ntp server.')
  else:
    logger.info('Synced system time with ntp server.')

def CallSysctl(logger, name, value):
  """Write a variable using sysctl.

  Args:
      logger: logger object, used to write to SysLog and serial port.
      name: string name of the sysctl variable.
      value: value of the sysctl variable.
  """
  logger.info('Configuring sysctl %s.', name)

  sysctl_command = [
      'sysctl', '-w', '{name}={value}'.format(name=name, value=value)]
  try:
    subprocess.check_call(sysctl_command)
  except subprocess.CalledProcessError:
    logger.warning('Unable to configure sysctl %s.', name)



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for utils.py module."""

from google_compute_engine.distro_lib.freebsd_11 import utils
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class UtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_setup = mock.create_autospec(utils.Utils)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallDhclient')
  def testEnableNetworkInterfaces(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['A', 'B'], self.mock_logger)
    expected_calls = [mock.call.call(['A', 'B'], mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallNtpdate')
  def testHandleClockSync(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.HandleClockSync(self.mock_setup, self.mock_logger)
    expected_calls = [mock.call.call(mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.IpForwardingUtilsIfconfig')
  def testIpForwardingUtils(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.IpForwardingUtils(self.mock_setup, self.mock_logger, 66)
    expected_calls = [mock.call.call(mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)



#!/usr/bin/python
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for provisioning or deprovisioning a Linux user account."""

import errno
import os
import subprocess
import time

from google_compute_engine import constants

NSS_CACHE_DURATION_SEC = 21600  # 6 hours in seconds.


class OsLoginUtils(object):
  """Utilities for OS Login activation."""

  def __init__(self, logger):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """
    self.logger = logger
    self.oslogin_installed = True
    self.update_time = 0

  def _RunOsLoginControl(self, params):
    """Run the OS Login control script.

    Args:
      params: list, the params to pass to the script

    Returns:
      int, the return code from the call, or None if the script is not found.
    """
    try:
      return subprocess.call([constants.OSLOGIN_CONTROL_SCRIPT] + params)
    except OSError as e:
      if e.errno == errno.ENOENT:
        return None
      else:
        raise

  def _GetStatus(self, two_factor=False):
    """Check whether OS Login is installed.

    Args:
      two_factor: bool, True if two factor should be enabled.

    Returns:
      bool, True if OS Login is installed.
    """
    params = ['status']
    if two_factor:
      params += ['--twofactor']
    retcode = self._RunOsLoginControl(params)
    if retcode is None:
      if self.oslogin_installed:
        self.logger.warning('OS Login not installed.')
        self.oslogin_installed = False
      return None

    # Prevent log spam when OS Login is not installed.
    self.oslogin_installed = True
    if not os.path.exists(constants.OSLOGIN_NSS_CACHE):
      return False
    return not retcode

  def _RunOsLoginNssCache(self):
    """Run the OS Login NSS cache binary.

    Returns:
      int, the return code from the call, or None if the script is not found.
    """
    try:
      return subprocess.call([constants.OSLOGIN_NSS_CACHE_SCRIPT])
    except OSError as e:
      if e.errno == errno.ENOENT:
        return None
      else:
        raise

  def _RemoveOsLoginNssCache(self):
    """Remove the OS Login NSS cache file."""
    if os.path.exists(constants.OSLOGIN_NSS_CACHE):
      try:
        os.remove(constants.OSLOGIN_NSS_CACHE)
      except OSError as e:
        if e.errno != errno.ENOENT:
          raise

  def UpdateOsLogin(self, oslogin_desired, two_factor_desired=False):
    """Update whether OS Login is enabled and update NSS cache if necessary.

    Args:
      oslogin_desired: bool, enable OS Login if True, disable if False.
      two_factor_desired: bool, enable two factor if True, disable if False.

    Returns:
      int, the return code from updating OS Login, or None if not present.
    """
    oslogin_configured = self._GetStatus(two_factor=False)
    if oslogin_configured is None:
      return None
    two_factor_configured = self._GetStatus(two_factor=True)
    # Two factor can only be enabled when OS Login is enabled.
    two_factor_desired = two_factor_desired and oslogin_desired

    if oslogin_desired:
      params = ['activate']
      if two_factor_desired:
        params += ['--twofactor']
      # OS Login is desired and not enabled.
      if not oslogin_configured:
        self.logger.info('Activating OS Login.')
        return self._RunOsLoginControl(params) or self._RunOsLoginNssCache()
      # Enable two factor authentication.
      if two_factor_desired and not two_factor_configured:
        self.logger.info('Activating OS Login two factor authentication.')
        return self._RunOsLoginControl(params) or self._RunOsLoginNssCache()
      # Deactivate two factor authentication.
      if two_factor_configured and not two_factor_desired:
        self.logger.info('Reactivating OS Login with two factor disabled.')
        return (self._RunOsLoginControl(['deactivate'])
                or self._RunOsLoginControl(params))
      # OS Login features are already enabled. Update the cache if appropriate.
      current_time = time.time()
      if current_time - self.update_time > NSS_CACHE_DURATION_SEC:
        self.update_time = current_time
        return self._RunOsLoginNssCache()

    elif oslogin_configured:
      self.logger.info('Deactivating OS Login.')
      return (self._RunOsLoginControl(['deactivate'])
              or self._RemoveOsLoginNssCache())

    # No action was needed.
    return 0



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Retrieve and store user provided metadata scripts."""

import functools
import re
import socket
import tempfile
import time

from google_compute_engine import metadata_watcher
from google_compute_engine.compat import httpclient
from google_compute_engine.compat import urlerror
from google_compute_engine.compat import urlrequest
from google_compute_engine.compat import urlretrieve


def _RetryOnUnavailable(func):
  """Function decorator template to retry on a service unavailable exception."""

  @functools.wraps(func)
  def Wrapper(*args, **kwargs):
    final_exception = None
    for _ in range(3):
      try:
        response = func(*args, **kwargs)
      except (httpclient.HTTPException, socket.error, urlerror.URLError) as e:
        final_exception = e
        time.sleep(5)
        continue
      else:
        return response
    raise final_exception
  return Wrapper


@_RetryOnUnavailable
def _UrlOpenWithRetry(request):
  """Call urlopen with retry."""
  return urlrequest.urlopen(request)


@_RetryOnUnavailable
def _UrlRetrieveWithRetry(url, dest):
  """Call urlretrieve with retry."""
  return urlretrieve.urlretrieve(url, dest)


class ScriptRetriever(object):
  """A class for retrieving and storing user provided metadata scripts."""
  token_metadata_key = 'instance/service-accounts/default/token'
  # Cached authentication token to be used when downloading from bucket.
  token = None

  def __init__(self, logger, script_type):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      script_type: string, the metadata script type to run.
    """
    self.logger = logger
    self.script_type = script_type
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)

  def _DownloadAuthUrl(self, url, dest_dir):
    """Download a Google Storage URL using an authentication token.

    If the token cannot be fetched, fallback to unauthenticated download.

    Args:
      url: string, the URL to download.
      dest_dir: string, the path to a directory for storing metadata scripts.

    Returns:
      string, the path to the file storing the metadata script.
    """
    dest_file = tempfile.NamedTemporaryFile(dir=dest_dir, delete=False)
    dest_file.close()
    dest = dest_file.name

    self.logger.info(
        'Downloading url from %s to %s using authentication token.', url, dest)

    if not self.token:
      response = self.watcher.GetMetadata(
          self.token_metadata_key, recursive=False, retry=False)

      if not response:
        self.logger.info(
            'Authentication token not found. Attempting unauthenticated '
            'download.')
        return self._DownloadUrl(url, dest_dir)

      self.token = '%s %s' % (
          response.get('token_type', ''), response.get('access_token', ''))

    try:
      request = urlrequest.Request(url)
      request.add_unredirected_header('Metadata-Flavor', 'Google')
      request.add_unredirected_header('Authorization', self.token)
      content = _UrlOpenWithRetry(request).read()
    except Exception as e:
      self.logger.warning('Could not download %s. %s.', url, str(e))
      return None

    with open(dest, 'wb') as f:
      f.write(content)

    return dest

  def _DownloadUrl(self, url, dest_dir):
    """Download a script from a given URL.

    Args:
      url: string, the URL to download.
      dest_dir: string, the path to a directory for storing metadata scripts.

    Returns:
      string, the path to the file storing the metadata script.
    """
    dest_file = tempfile.NamedTemporaryFile(dir=dest_dir, delete=False)
    dest_file.close()
    dest = dest_file.name

    self.logger.info('Downloading url from %s to %s.', url, dest)
    try:
      _UrlRetrieveWithRetry(url, dest)
      return dest
    except (httpclient.HTTPException, socket.error, urlerror.URLError) as e:
      self.logger.warning('Could not download %s. %s.', url, str(e))
    except Exception as e:
      self.logger.warning('Exception downloading %s. %s.', url, str(e))
    return None

  def _DownloadScript(self, url, dest_dir):
    """Download the contents of the URL to the destination.

    Args:
      url: string, the URL to download.
      dest_dir: string, the path to a directory for storing metadata scripts.

    Returns:
      string, the path to the file storing the metadata script.
    """
    # Check for the preferred Google Storage URL format:
    # gs://<bucket>/<object>
    if url.startswith(r'gs://'):
      # Convert the string into a standard URL.
      url = re.sub('^gs://', 'https://storage.googleapis.com/', url)
      return self._DownloadAuthUrl(url, dest_dir)

    header = r'http[s]?://'
    domain = r'storage\.googleapis\.com'

    # Many of the Google Storage URLs are supported below.
    # It is prefered that customers specify their object using
    # its gs://<bucket>/<object> url.
    bucket = r'(?P<bucket>[a-z0-9][-_.a-z0-9]*[a-z0-9])'

    # Accept any non-empty string that doesn't contain a wildcard character
    obj = r'(?P<obj>[^\*\?]+)'

    # Check for the Google Storage URLs:
    # http://<bucket>.storage.googleapis.com/<object>
    # https://<bucket>.storage.googleapis.com/<object>
    gs_regex = re.compile(r'\A%s%s\.%s/%s\Z' % (header, bucket, domain, obj))
    match = gs_regex.match(url)
    if match:
      return self._DownloadAuthUrl(url, dest_dir)

    # Check for the other possible Google Storage URLs:
    # http://storage.googleapis.com/<bucket>/<object>
    # https://storage.googleapis.com/<bucket>/<object>
    #
    # The following are deprecated but checked:
    # http://commondatastorage.googleapis.com/<bucket>/<object>
    # https://commondatastorage.googleapis.com/<bucket>/<object>
    gs_regex = re.compile(
        r'\A%s(commondata)?%s/%s/%s\Z' % (header, domain, bucket, obj))
    match = gs_regex.match(url)
    if match:
      return self._DownloadAuthUrl(url, dest_dir)

    # Unauthenticated download of the object.
    return self._DownloadUrl(url, dest_dir)

  def _GetAttributeScripts(self, attribute_data, dest_dir):
    """Retrieve the scripts from attribute metadata.

    Args:
      attribute_data: dict, the contents of the attributes metadata.
      dest_dir: string, the path to a directory for storing metadata scripts.

    Returns:
      dict, a dictionary mapping metadata keys to files storing scripts.
    """
    script_dict = {}
    attribute_data = attribute_data or {}
    metadata_key = '%s-script' % self.script_type
    metadata_value = attribute_data.get(metadata_key)
    if metadata_value:
      self.logger.info('Found %s in metadata.', metadata_key)
      with tempfile.NamedTemporaryFile(
          mode='w', dir=dest_dir, delete=False) as dest:
        dest.write(metadata_value.lstrip())
        script_dict[metadata_key] = dest.name

    metadata_key = '%s-script-url' % self.script_type
    metadata_value = attribute_data.get(metadata_key)
    if metadata_value:
      self.logger.info('Found %s in metadata.', metadata_key)
      downloaded_dest = self._DownloadScript(metadata_value, dest_dir)
      if downloaded_dest is None:
        self.logger.warning('Failed to download metadata script.')
      script_dict[metadata_key] = downloaded_dest

    return script_dict

  def GetScripts(self, dest_dir):
    """Retrieve the scripts to execute.

    Args:
      dest_dir: string, the path to a directory for storing metadata scripts.

    Returns:
      dict, a dictionary mapping set metadata keys with associated scripts.
    """
    metadata_dict = self.watcher.GetMetadata() or {}

    try:
      instance_data = metadata_dict['instance']['attributes']
    except KeyError:
      instance_data = None
      self.logger.warning('Instance attributes were not found.')

    try:
      project_data = metadata_dict['project']['attributes']
    except KeyError:
      project_data = None
      self.logger.warning('Project attributes were not found.')

    return (self._GetAttributeScripts(instance_data, dest_dir)
            or self._GetAttributeScripts(project_data, dest_dir))



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities that are distro specific for use on EL 7."""

import fileinput
import os
import re

from google_compute_engine import constants
from google_compute_engine.distro_lib import helpers
from google_compute_engine.distro_lib import ip_forwarding_utils
from google_compute_engine.distro_lib import utils


class Utils(utils.Utils):
  """Utilities used by Linux guest services on EL 7."""

  network_path = constants.LOCALBASE + '/etc/sysconfig/network-scripts'

  def EnableIpv6(self, interfaces, logger, dhclient_script=None):
    """Configure the network interfaces for IPv6 using dhclient.

    Args:
      interface: string, the output device names for enabling IPv6.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    helpers.CallEnableRouteAdvertisements(interfaces, logger)
    helpers.CallDhclientIpv6(interfaces, logger)

  def DisableIpv6(self, interfaces, logger):
    """Disable Ipv6 by giving up the DHCP lease using dhclient.

    Args:
      interface: string, the output device names for enabling IPv6.
      logger: logger object, used to write to SysLog and serial port.
    """
    helpers.CallDhclientIpv6(interfaces, logger, None, release_lease=True)

  def EnableNetworkInterfaces(self, interfaces, logger, dhclient_script=None):
    """Enable the list of network interfaces.

    Args:
      interfaces: list of string, the output device names to enable.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    # Should always exist in EL 7.
    if os.path.exists(self.network_path):
      self._DisableNetworkManager(interfaces, logger)
    helpers.CallDhclient(interfaces, logger)

  def _DisableNetworkManager(self, interfaces, logger):
    """Disable network manager management on a list of network interfaces.

    Args:
      interfaces: list of string, the output device names enable.
      logger: logger object, used to write to SysLog and serial port.
    """
    for interface in interfaces:
      interface_config = os.path.join(
          self.network_path, 'ifcfg-%s' % interface)
      if os.path.exists(interface_config):
        self._ModifyInterface(
            interface_config, 'DEVICE', interface, replace=False)
        self._ModifyInterface(
            interface_config, 'NM_CONTROLLED', 'no', replace=True)
      else:
        with open(interface_config, 'w') as interface_file:
          interface_content = [
              '# Added by Google.',
              'BOOTPROTO=none',
              'DEFROUTE=no',
              'DEVICE=%s' % interface,
              'IPV6INIT=no',
              'NM_CONTROLLED=no',
              'NOZEROCONF=yes',
              '',
          ]
          interface_file.write('\n'.join(interface_content))
        logger.info('Created config file for interface %s.', interface)

  def _ModifyInterface(
      self, interface_config, config_key, config_value, replace=False):
    """Write a value to a config file if not already present.

    Args:
      interface_config: string, the path to a config file.
      config_key: string, the configuration key to set.
      config_value: string, the value to set for the configuration key.
      replace: bool, replace the configuration option if already present.
    """
    config_entry = '%s=%s' % (config_key, config_value)
    if not open(interface_config).read().count(config_key):
      with open(interface_config, 'a') as config:
        config.write('%s\n' % config_entry)
    elif replace:
      for line in fileinput.input(interface_config, inplace=True):
        print(re.sub(r'%s=.*' % config_key, config_entry, line.rstrip()))

  def HandleClockSync(self, logger):
    """Sync the software clock with the hypervisor clock.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """
    helpers.CallHwclock(logger)

  def IpForwardingUtils(self, logger, proto_id=None):
    """Get system IP address configuration utilities.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      proto_id: string, the routing protocol identifier for Google IP changes.
    """
    return ip_forwarding_utils.IpForwardingUtilsIproute(logger, proto_id)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Enables the network interfaces for multi-nic support."""

import logging.handlers
import subprocess

from google_compute_engine import logger
from google_compute_engine.compat import distro_utils


class NetworkSetup(object):
  """Enable network interfaces."""

  interfaces = set()
  network_interfaces = 'instance/network-interfaces'

  def __init__(self, dhclient_script=None, dhcp_command=None, debug=False):
    """Constructor.

    Args:
      dhclient_script: string, the path to a dhclient script used by dhclient.
      dhcp_command: string, a command to enable Ethernet interfaces.
      debug: bool, True if debug output should write to the console.
    """
    self.dhclient_script = dhclient_script or '/sbin/google-dhclient-script'
    self.dhcp_command = dhcp_command
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='network-setup', debug=debug, facility=facility)
    self.distro_utils = distro_utils.Utils(debug=debug)
    self.ipv6_initialized = False
    self.ipv6_interfaces = set()

  def EnableIpv6(self, interfaces):
    """Enable IPv6 on the list of network interfaces.

    Args:
      interfaces: list of string, the output device names for enabling IPv6.
    """
    if not interfaces or self.ipv6_interfaces == set(interfaces):
      return

    self.logger.info('Enabling IPv6 on Ethernet interface: %s.', interfaces)
    self.ipv6_interfaces = self.ipv6_interfaces.union(set(interfaces))
    self.ipv6_initialized = True

    # Distro-specific setup for enabling IPv6 on network interfaces.
    self.distro_utils.EnableIpv6(
        interfaces, self.logger, dhclient_script=self.dhclient_script)

  def DisableIpv6(self, interfaces):
    """Disable IPv6 on the list of network interfaces.

    Args:
      interfaces: list of string, the output device names for disabling IPv6.
    """
    # Allow to run once during Initialization and after that only when an
    # interface is found in the ipv6_interfaces set.
    if not interfaces or (
        self.ipv6_initialized and not self.ipv6_interfaces.intersection(
            set(interfaces))):
      return

    self.logger.info('Disabling IPv6 on Ethernet interface: %s.', interfaces)
    self.ipv6_interfaces.difference_update(interfaces)
    self.ipv6_initialized = True

    # Distro-specific setup for disabling IPv6 on network interfaces.
    self.distro_utils.DisableIpv6(interfaces, self.logger)

  def EnableNetworkInterfaces(self, interfaces):
    """Enable the list of network interfaces.

    Args:
      interfaces: list of string, the output device names to enable.
    """
    # The default Ethernet interface is enabled by default. Do not attempt to
    # enable interfaces if only one interface is specified in metadata.
    if not interfaces or set(interfaces) == self.interfaces:
      return

    self.logger.info('Ethernet interfaces: %s.', interfaces)
    self.interfaces = set(interfaces)

    if self.dhcp_command:
      try:
        subprocess.check_call([self.dhcp_command])
      except subprocess.CalledProcessError:
        self.logger.warning('Could not enable Ethernet interfaces.')
      return

    # Distro-specific setup for network interfaces.
    self.distro_utils.EnableNetworkInterfaces(
        interfaces, self.logger, dhclient_script=self.dhclient_script)



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unittest for utils.py module."""

import subprocess

from google_compute_engine.distro_lib.sles_12 import utils
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class UtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_setup = mock.create_autospec(utils.Utils)
    self.mock_setup.network_path = '/etc/sysconfig/network'

  def testEnableNetworkInterfacesWithSingleNic(self):
    mocks = mock.Mock()

    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['eth0'], self.mock_logger)
    expected_calls = []
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testEnableNetworkInterfacesWithMultipleNics(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_setup._WriteIfcfg, 'writeIfcfg')
    mocks.attach_mock(self.mock_setup._Ifup, 'ifup')

    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['eth0', 'eth1', 'eth2'], self.mock_logger)
    expected_calls = [
        mock.call.writeIfcfg(['eth1', 'eth2'], mock.ANY),
        mock.call.ifup(['eth1', 'eth2'], mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testWriteIfcfg(self):
    mocks = mock.Mock()
    mock_open = mock.mock_open()
    mocks.attach_mock(mock_open, 'open')
    with mock.patch('%s.open' % builtin, mock_open, create=False):

      utils.Utils._WriteIfcfg(
          self.mock_setup, ['eth1', 'eth2'], self.mock_logger)
      expected_calls = [
          mock.call.open('/etc/sysconfig/network/ifcfg-eth1', 'w'),
          mock.call.open().__enter__(),
          mock.call.open().write(mock.ANY),
          mock.call.open().__exit__(None, None, None),
          mock.call.open('/etc/sysconfig/network/ifcfg-eth2', 'w'),
          mock.call.open().__enter__(),
          mock.call.open().write(mock.ANY),
          mock.call.open().__exit__(None, None, None),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch(
      'google_compute_engine.distro_lib.sles_12.utils.subprocess.check_call')
  def testIfup(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')
    mock_call.side_effect = [
        None, subprocess.CalledProcessError(1, 'Test'),
    ]

    utils.Utils._Ifup(self.mock_setup, ['eth1', 'eth2'], self.mock_logger)
    utils.Utils._Ifup(self.mock_setup, ['eth1', 'eth2'], self.mock_logger)
    expectedIfupCall = [
        '/usr/sbin/wicked', 'ifup', '--timeout', '1', 'eth1', 'eth2',
    ]
    expected_calls = [
        mock.call.call(expectedIfupCall),
        mock.call.call(expectedIfupCall),
        mock.call.logger.warning(mock.ANY, ['eth1', 'eth2']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallHwclock')
  def testHandleClockSync(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.HandleClockSync(self.mock_setup, self.mock_logger)
    expected_calls = [mock.call.call(mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.IpForwardingUtilsIproute')
  def testIpForwardingUtils(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.IpForwardingUtils(self.mock_setup, self.mock_logger, '66')
    expected_calls = [mock.call.call(mock.ANY, '66')]
    self.assertEqual(mocks.mock_calls, expected_calls)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for compute_auth.py module."""

import sys

from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest

if sys.version_info < (3, 0):
  from google_compute_engine.boto import compute_auth


@unittest.skipIf(sys.version_info > (3, 0), 'Skipping for python3.')
class ComputeAuthTest(unittest.TestCase):

  def setUp(self):
    self.metadata_key = 'instance/service-accounts'
    self.service_account = 'service_account'
    self.mock_config = mock.Mock()
    self.mock_config.get.return_value = self.service_account
    self.mock_provider = mock.Mock()
    self.mock_provider.name = 'google'

  @mock.patch('google_compute_engine.boto.compute_auth.metadata_watcher')
  @mock.patch('google_compute_engine.boto.compute_auth.logger')
  def testCreateConfig(self, mock_logger, mock_watcher):
    scopes = list(compute_auth.GS_SCOPES)[1:2]
    service_accounts = {self.service_account: {'scopes': scopes}}
    mock_watcher.GetMetadata.return_value = service_accounts
    mock_watcher.MetadataWatcher.return_value = mock_watcher
    mocks = mock.Mock()
    mocks.attach_mock(mock_watcher, 'watcher')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(self.mock_config, 'config')
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance

    mock_compute_auth = compute_auth.ComputeAuth(
        None, self.mock_config, self.mock_provider)
    expected_calls = [
        mock.call.logger.Logger(name=mock.ANY),
        mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
        mock.call.config.get('GoogleCompute', 'service_account', ''),
        mock.call.watcher.GetMetadata(metadata_key=self.metadata_key)
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)
    self.assertEqual(mock_compute_auth.scopes, scopes)

  @mock.patch('google_compute_engine.boto.compute_auth.metadata_watcher')
  def testCreateConfigNoScopes(self, mock_watcher):
    mock_watcher.GetMetadata.return_value = {}
    mock_watcher.MetadataWatcher.return_value = mock_watcher

    with self.assertRaises(compute_auth.auth_handler.NotReadyToAuthenticate):
      compute_auth.ComputeAuth(None, self.mock_config, self.mock_provider)

  def testCreateConfigNoServiceAccount(self):
    self.mock_config.get.return_value = None

    with self.assertRaises(compute_auth.auth_handler.NotReadyToAuthenticate):
      compute_auth.ComputeAuth(None, self.mock_config, self.mock_provider)

  @mock.patch('google_compute_engine.boto.compute_auth.metadata_watcher')
  def testGetAccessToken(self, mock_watcher):
    mock_auth = mock.create_autospec(compute_auth.ComputeAuth)
    mock_auth.watcher = mock_watcher
    mock_auth.metadata_key = self.metadata_key
    mock_auth.service_account = self.service_account
    mock_watcher.GetMetadata.side_effect = [
        {self.service_account: {'token': {'access_token': 'test'}}},
        {},
    ]

    self.assertEqual(
        compute_auth.ComputeAuth._GetAccessToken(mock_auth), 'test')
    self.assertEqual(
        compute_auth.ComputeAuth._GetAccessToken(mock_auth), None)
    expected_calls = [mock.call(metadata_key=self.metadata_key)] * 2
    self.assertEqual(mock_watcher.GetMetadata.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.boto.compute_auth.metadata_watcher')
  def testAddAuth(self, mock_watcher):
    mock_auth = mock.create_autospec(compute_auth.ComputeAuth)
    mock_auth._GetAccessToken.return_value = 'token'
    mock_request = mock.Mock()
    mock_request.headers = {}

    compute_auth.ComputeAuth.add_auth(mock_auth, mock_request)
    self.assertEqual(mock_request.headers['Authorization'], 'OAuth token')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Manage IP forwarding on a Google Compute Engine instance.

When given a list of public endpoint IPs, compare it with the IPs configured
for the associated interfaces, and add or remove addresses from the interfaces
to make them match.
"""

import logging.handlers

from google_compute_engine import logger
from google_compute_engine.networking.ip_forwarding import ip_forwarding_utils


class IpForwarding(object):
  """Manage IP forwarding based on changes to forwarded IPs metadata."""

  def __init__(self, proto_id=None, debug=False):
    """Constructor.

    Args:
      proto_id: string, the routing protocol identifier for Google IP changes.
      debug: bool, True if debug output should write to the console.
    """
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(
        name='google-ip-forwarding', debug=debug, facility=facility)
    self.ip_forwarding_utils = ip_forwarding_utils.IpForwardingUtils(
        logger=self.logger, proto_id=proto_id)

  def _LogForwardedIpChanges(
      self, configured, desired, to_add, to_remove, interface):
    """Log the planned IP address changes.

    Args:
      configured: list, the IP address strings already configured.
      desired: list, the IP address strings that will be configured.
      to_add: list, the forwarded IP address strings to configure.
      to_remove: list, the forwarded IP address strings to delete.
      interface: string, the output device to modify.
    """
    if not to_add and not to_remove:
      return
    self.logger.info(
        'Changing %s IPs from %s to %s by adding %s and removing %s.',
        interface, configured or None, desired or None, to_add or None,
        to_remove or None)

  def _AddForwardedIps(self, forwarded_ips, interface):
    """Configure the forwarded IP address on the network interface.

    Args:
      forwarded_ips: list, the forwarded IP address strings to configure.
      interface: string, the output device to use.
    """
    for address in forwarded_ips:
      self.ip_forwarding_utils.AddForwardedIp(address, interface)

  def _RemoveForwardedIps(self, forwarded_ips, interface):
    """Remove the forwarded IP addresses from the network interface.

    Args:
      forwarded_ips: list, the forwarded IP address strings to delete.
      interface: string, the output device to use.
    """
    for address in forwarded_ips:
      self.ip_forwarding_utils.RemoveForwardedIp(address, interface)

  def HandleForwardedIps(self, interface, forwarded_ips, interface_ip=None):
    """Handle changes to the forwarded IPs on a network interface.

    Args:
      interface: string, the output device to configure.
      forwarded_ips: list, the forwarded IP address strings desired.
      interface_ip: string, current interface ip address.
    """
    desired = self.ip_forwarding_utils.ParseForwardedIps(forwarded_ips)
    configured = self.ip_forwarding_utils.GetForwardedIps(
        interface, interface_ip)
    to_add = sorted(set(desired) - set(configured))
    to_remove = sorted(set(configured) - set(desired))
    self._LogForwardedIpChanges(
        configured, desired, to_add, to_remove, interface)
    self._AddForwardedIps(to_add, interface)
    self._RemoveForwardedIps(to_remove, interface)



#!/usr/bin/python
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for oslogin_utils.py module."""

import itertools

from google_compute_engine.accounts import oslogin_utils
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class OsLoginUtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.oslogin_control_script = 'google_oslogin_control'
    self.oslogin_nss_cache = '/etc/oslogin_passwd.cache'
    self.oslogin_nss_cache_script = 'google_oslogin_nss_cache'

    self.mock_oslogin = mock.create_autospec(oslogin_utils.OsLoginUtils)
    self.mock_oslogin.logger = self.mock_logger
    self.mock_oslogin.oslogin_installed = True
    self.mock_oslogin.update_time = 0

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testRunOsLoginControl(self, mock_call):
    expected_return_value = 0
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.return_value = expected_return_value

    self.assertEqual(
        oslogin_utils.OsLoginUtils._RunOsLoginControl(
            self.mock_oslogin, ['activate']), expected_return_value)
    expected_calls = [
        mock.call.call([self.oslogin_control_script, 'activate']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testRunOsLoginControlStatus(self, mock_call):
    expected_return_value = 3
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.return_value = expected_return_value

    self.assertEqual(
        oslogin_utils.OsLoginUtils._RunOsLoginControl(
            self.mock_oslogin, ['status']), expected_return_value)
    expected_calls = [
        mock.call.call([self.oslogin_control_script, 'status']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testOsLoginNotInstalled(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.side_effect = OSError(2, 'Not Found')

    self.assertIsNone(
        oslogin_utils.OsLoginUtils._RunOsLoginControl(
            self.mock_oslogin, ['status']))
    expected_calls = [
        mock.call.call([self.oslogin_control_script, 'status']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testOsLoginControlError(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.side_effect = OSError

    with self.assertRaises(OSError):
      oslogin_utils.OsLoginUtils._RunOsLoginControl(
          self.mock_oslogin, ['status'])
    expected_calls = [
        mock.call.call([self.oslogin_control_script, 'status']),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testGetStatusActive(self, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_oslogin._RunOsLoginControl.return_value = 0
    mock_exists.return_value = True

    self.assertTrue(
        oslogin_utils.OsLoginUtils._GetStatus(
            self.mock_oslogin, two_factor=False))
    expected_calls = [
        mock.call.oslogin._RunOsLoginControl(['status']),
        mock.call.exists(self.oslogin_nss_cache),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testGetStatusNotActive(self, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_oslogin._RunOsLoginControl.return_value = 3
    mock_exists.return_value = True

    self.assertFalse(
        oslogin_utils.OsLoginUtils._GetStatus(
            self.mock_oslogin, two_factor=True))
    expected_calls = [
        mock.call.oslogin._RunOsLoginControl(['status', '--twofactor']),
        mock.call.exists(self.oslogin_nss_cache),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testGetStatusNoCache(self, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    self.mock_oslogin._RunOsLoginControl.return_value = 0
    mock_exists.return_value = False

    self.assertFalse(oslogin_utils.OsLoginUtils._GetStatus(self.mock_oslogin))
    expected_calls = [mock.call.exists(self.oslogin_nss_cache)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testGetStatusNotInstalled(self):
    mocks = mock.Mock()
    self.mock_oslogin._RunOsLoginControl.return_value = None
    mocks.attach_mock(self.mock_logger, 'logger')

    self.assertTrue(self.mock_oslogin.oslogin_installed)
    self.assertFalse(oslogin_utils.OsLoginUtils._GetStatus(self.mock_oslogin))
    self.assertFalse(self.mock_oslogin.oslogin_installed)
    self.assertFalse(oslogin_utils.OsLoginUtils._GetStatus(self.mock_oslogin))
    # Should only log once, even though called twice.
    expected_calls = [
        mock.call.logger.warning(mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testRunOsLoginNssCache(self, mock_call):
    expected_return_value = 0
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.return_value = expected_return_value

    self.assertEqual(
        oslogin_utils.OsLoginUtils._RunOsLoginNssCache(self.mock_oslogin),
        expected_return_value)
    expected_calls = [
        mock.call.call([self.oslogin_nss_cache_script]),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testRunOsLoginNssCacheNotInstalled(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.side_effect = OSError(2, 'Not Found')

    self.assertIsNone(
        oslogin_utils.OsLoginUtils._RunOsLoginNssCache(self.mock_oslogin))
    expected_calls = [
        mock.call.call([self.oslogin_nss_cache_script]),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.subprocess.call')
  def testRunOsLoginNssCacheError(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mock_call.side_effect = OSError

    with self.assertRaises(OSError):
      oslogin_utils.OsLoginUtils._RunOsLoginNssCache(self.mock_oslogin)
    expected_calls = [
        mock.call.call([self.oslogin_nss_cache_script]),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testRemoveOsLoginNssCache(self, mock_exists, mock_remove):
    mock_exists.return_value = True

    oslogin_utils.OsLoginUtils._RemoveOsLoginNssCache(self.mock_oslogin)
    mock_exists.assert_called_once_with(self.oslogin_nss_cache)
    mock_remove.assert_called_once_with(self.oslogin_nss_cache)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testRemoveOsLoginNssCacheNotFound(self, mock_exists, mock_remove):
    mock_exists.return_value = False

    oslogin_utils.OsLoginUtils._RemoveOsLoginNssCache(self.mock_oslogin)
    mock_exists.assert_called_once_with(self.oslogin_nss_cache)
    mock_remove.assert_not_called()

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testRemoveOsLoginNssCacheNotInstalled(self, mock_exists, mock_remove):
    mock_exists.return_value = True
    mock_remove.side_effect = OSError(2, 'Not Found')

    oslogin_utils.OsLoginUtils._RemoveOsLoginNssCache(self.mock_oslogin)
    mock_exists.assert_called_once_with(self.oslogin_nss_cache)
    mock_remove.assert_called_once_with(self.oslogin_nss_cache)

  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.oslogin_utils.os.path.exists')
  def testRemoveOsLoginNssCacheError(self, mock_exists, mock_remove):
    mock_exists.return_value = True
    mock_remove.side_effect = OSError

    with self.assertRaises(OSError):
      oslogin_utils.OsLoginUtils._RemoveOsLoginNssCache(self.mock_oslogin)

  @mock.patch('time.time')
  def testUpdateOsLoginUpdateCache(self, mock_time):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_oslogin._RunOsLoginControl.return_value = 0
    self.mock_oslogin._GetStatus.return_value = True
    mock_time.return_value = 6 * 60 * 60 + 1

    oslogin_utils.OsLoginUtils.UpdateOsLogin(
        self.mock_oslogin, True, two_factor_desired=True)
    expected_calls = [
        mock.call.oslogin._GetStatus(two_factor=False),
        mock.call.oslogin._GetStatus(two_factor=True),
        mock.call.oslogin._RunOsLoginNssCache(),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('time.time')
  def testUpdateOsLogin(self, mock_time):

    def _AssertNoUpdate():
      expected_calls = [
          mock.call.oslogin._GetStatus(two_factor=False),
          mock.call.oslogin._GetStatus(two_factor=True),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)
      self.assertEqual(return_value, 0)

    def _AssertActivated(two_factor=False):
      params = ['activate', '--twofactor'] if two_factor else ['activate']
      expected_calls = [
          mock.call.oslogin._GetStatus(two_factor=False),
          mock.call.oslogin._GetStatus(two_factor=True),
          mock.call.logger.info(mock.ANY),
          mock.call.oslogin._RunOsLoginControl(params),
          mock.call.oslogin._RunOsLoginNssCache(),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

    def _AssertDeactivated():
      expected_calls = [
          mock.call.oslogin._GetStatus(two_factor=False),
          mock.call.oslogin._GetStatus(two_factor=True),
          mock.call.logger.info(mock.ANY),
          mock.call.oslogin._RunOsLoginControl(['deactivate']),
          mock.call.oslogin._RemoveOsLoginNssCache(),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

    def _AssertReactivated():
      expected_calls = [
          mock.call.oslogin._GetStatus(two_factor=False),
          mock.call.oslogin._GetStatus(two_factor=True),
          mock.call.logger.info(mock.ANY),
          mock.call.oslogin._RunOsLoginControl(['deactivate']),
          mock.call.oslogin._RunOsLoginControl(['activate']),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

    parameters = list(itertools.product([False, True], repeat=4))
    for (oslogin, two_factor, oslogin_config, two_factor_config) in parameters:
      mocks = mock.Mock()
      mocks.attach_mock(self.mock_logger, 'logger')
      mocks.attach_mock(self.mock_oslogin, 'oslogin')
      self.mock_oslogin._RunOsLoginControl.return_value = 0
      self.mock_oslogin._GetStatus.side_effect = [
          oslogin_config, two_factor_config]
      mock_time.return_value = 6 * 60 * 60
      return_value = oslogin_utils.OsLoginUtils.UpdateOsLogin(
          self.mock_oslogin, oslogin, two_factor_desired=two_factor)

      if oslogin_config:
        if not oslogin:
          _AssertDeactivated()
        elif two_factor_config:
          if not two_factor:
            _AssertReactivated()
          else:
            _AssertNoUpdate()
        else:
          if two_factor:
            _AssertActivated(two_factor=True)
          else:
            _AssertNoUpdate()
      else:
        if oslogin:
          _AssertActivated(two_factor=two_factor)
        else:
          _AssertNoUpdate()
      self.mock_logger.reset_mock()
      self.mock_oslogin.reset_mock()

  def testUpdateOsLoginNotInstalled(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_oslogin._RunOsLoginControl.return_value = 0
    self.mock_oslogin._GetStatus.return_value = None

    return_value = oslogin_utils.OsLoginUtils.UpdateOsLogin(
        self.mock_oslogin, True)
    expected_calls = [mock.call.oslogin._GetStatus(two_factor=False)]
    self.assertEqual(mocks.mock_calls, expected_calls)
    self.assertEqual(return_value, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for script_executor.py module."""

import stat

from google_compute_engine.metadata_scripts import script_executor
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class ScriptExecutorTest(unittest.TestCase):

  def setUp(self):
    self.script_type = 'test'
    self.metadata_script = '/tmp/script'
    self.mock_logger = mock.Mock()
    self.executor = script_executor.ScriptExecutor(
        self.mock_logger, self.script_type)

  @mock.patch('google_compute_engine.metadata_scripts.script_executor.os')
  def testMakeExecutable(self, mock_os):
    st_mode = 1
    chmod_mode = st_mode + stat.S_IEXEC
    mock_os_stat = mock.Mock()
    mock_os_stat.st_mode = st_mode
    mock_os.stat.return_value = mock_os_stat
    self.executor._MakeExecutable(self.metadata_script)
    mock_os.chmod.assert_called_once_with(self.metadata_script, chmod_mode)

  @mock.patch('google_compute_engine.metadata_scripts.script_executor.subprocess')
  def testRunScript(self, mock_subprocess):
    mock_readline = mock.Mock()
    mock_readline.side_effect = [bytes(b'a\n'), bytes(b'b\n'), bytes(b'')]
    mock_stdout = mock.Mock()
    mock_stdout.readline = mock_readline
    mock_process = mock.Mock()
    mock_process.poll.return_value = 0
    mock_process.stdout = mock_stdout
    mock_process.returncode = 1
    mock_subprocess.Popen.return_value = mock_process
    metadata_key = '%s-script' % self.script_type

    self.executor._RunScript(metadata_key, self.metadata_script)
    expected_calls = [
        mock.call('%s: %s', metadata_key, 'a'),
        mock.call('%s: %s', metadata_key, 'b'),
        mock.call('%s: Return code %s.', metadata_key, 1),
    ]
    self.assertEqual(self.mock_logger.info.mock_calls, expected_calls)
    mock_subprocess.Popen.assert_called_once_with(
        self.metadata_script, shell=True, executable='/bin/bash',
        stderr=mock_subprocess.STDOUT, stdout=mock_subprocess.PIPE)
    mock_process.poll.assert_called_once_with()

  def testRunScripts(self):
    self.executor._MakeExecutable = mock.Mock()
    self.executor._RunScript = mock.Mock()
    mocks = mock.Mock()
    mocks.attach_mock(self.executor._MakeExecutable, 'make_executable')
    mocks.attach_mock(self.executor._RunScript, 'run_script')
    mocks.attach_mock(self.mock_logger, 'logger')
    script_dict = {
        '%s-script' % self.script_type: 'a',
        '%s-script-key' % self.script_type: 'b',
        '%s-script-url' % self.script_type: 'c',
    }

    self.executor.RunScripts(script_dict)
    expected_calls = [
        mock.call.make_executable('c'),
        mock.call.run_script('%s-script-url' % self.script_type, 'c'),
        mock.call.make_executable('a'),
        mock.call.run_script('%s-script' % self.script_type, 'a'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testRunScriptsEmpty(self):
    self.executor._MakeExecutable = mock.Mock()
    self.executor._RunScript = mock.Mock()
    mocks = mock.Mock()
    mocks.attach_mock(self.executor._MakeExecutable, 'make_executable')
    mocks.attach_mock(self.executor._RunScript, 'run_script')
    mocks.attach_mock(self.mock_logger, 'logger')
    script_dict = {
        '%s-invalid' % self.script_type: 'script',
    }

    self.executor.RunScripts(script_dict)
    expected_calls = [
        mock.call.logger.info(mock.ANY, self.script_type),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module for global constants."""

import platform

OSLOGIN_CONTROL_SCRIPT = 'google_oslogin_control'
OSLOGIN_NSS_CACHE_SCRIPT = 'google_oslogin_nss_cache'

if platform.system() == 'FreeBSD':
  BOTOCONFDIR = '/usr/local'
  LOCALBASE = '/usr/local'
  LOCALSTATEDIR = '/var/spool'
  OSLOGIN_NSS_CACHE = '/usr/local/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/usr/local/etc'
  SYSLOG_SOCKET = '/var/run/log'
elif platform.system() == 'OpenBSD':
  BOTOCONFDIR = ''
  LOCALBASE = '/usr/local'
  LOCALSTATEDIR = '/var/spool'
  OSLOGIN_NSS_CACHE = '/usr/local/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/usr/local/etc'
  SYSLOG_SOCKET = '/dev/log'
else:
  BOTOCONFDIR = ''
  LOCALBASE = ''
  LOCALSTATEDIR = '/var'
  OSLOGIN_NSS_CACHE = '/etc/oslogin_passwd.cache'
  SYSCONFDIR = '/etc/default'
  SYSLOG_SOCKET = '/dev/log'



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library used to set up the system boto.cfg file.

If a project ID is not provided, this request the project ID from the
metadata server and install the compute authentication plugin.

Note the config starts with the content in /etc/boto.cfg.template,
overrides settings, and then persists it into /etc/boto.cfg. This
is done so that the system boto.cfg can be removed prior to image
packaging.
"""

import os

from google_compute_engine import config_manager
from google_compute_engine import constants
from google_compute_engine import logger
from google_compute_engine import metadata_watcher


class BotoConfig(object):
  """Creates a boto config file for standalone GSUtil."""

  # WARNING: The path should remain as /etc/boto.cfg in order to not break
  # tools, such as gsutil, that rely on loading well-known Boto config paths.
  # If you want to change this, please consult the gsutil team
  # (GoogleCloudPlatform/gsutil) first.
  boto_config = constants.BOTOCONFDIR + '/etc/boto.cfg'
  boto_config_template = constants.BOTOCONFDIR + '/etc/boto.cfg.template'
  boto_config_script = os.path.abspath(__file__)
  boto_config_header = (
      'This file is automatically created at boot time by the %s script. Do '
      'not edit this file directly. If you need to add items to this file, '
      'create or edit %s instead and then re-run google_instance_setup.')

  def __init__(self, project_id=None, debug=False):
    """Constructor.

    Args:
      project_id: string, the project ID to use in the config file.
      debug: bool, True if debug output should write to the console.
    """
    self.logger = logger.Logger(name='boto-setup', debug=debug)
    self.watcher = metadata_watcher.MetadataWatcher(logger=self.logger)
    self._CreateConfig(project_id)

  def _GetNumericProjectId(self):
    """Get the numeric project ID for this VM.

    Returns:
      string, the numeric project ID if one is found.
    """
    project_id = 'project/numeric-project-id'
    return self.watcher.GetMetadata(metadata_key=project_id, recursive=False)

  def _CreateConfig(self, project_id):
    """Create the boto config to support standalone GSUtil.

    Args:
      project_id: string, the project ID to use in the config file.
    """
    project_id = project_id or self._GetNumericProjectId()

    # Our project doesn't support service accounts.
    if not project_id:
      return

    self.boto_config_header %= (
        self.boto_config_script, self.boto_config_template)
    config = config_manager.ConfigManager(
        config_file=self.boto_config_template,
        config_header=self.boto_config_header)

    # WARNING: If you want to change the contents of this config file, please
    # consult the gsutil team (GoogleCloudPlatform/gsutil) first.
    config.SetOption('GSUtil', 'default_project_id', project_id)
    config.SetOption('GSUtil', 'default_api_version', '2')
    config.SetOption('GoogleCompute', 'service_account', 'default')
    config.WriteConfig(config_file=self.boto_config)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library providing file utilities for setting permissions and locking."""

import contextlib
import errno
import fcntl
import os
import subprocess


def _SetSELinuxContext(path):
  """Set the appropriate SELinux context, if SELinux tools are installed.

  Calls /sbin/restorecon on the provided path to set the SELinux context as
  specified by policy. This call does not operate recursively.

  Only some OS configurations use SELinux. It is therefore acceptable for
  restorecon to be missing, in which case we do nothing.

  Args:
    path: string, the path on which to fix the SELinux context.
  """
  restorecon = '/sbin/restorecon'
  if os.path.isfile(restorecon) and os.access(restorecon, os.X_OK):
    subprocess.call([restorecon, path])


def SetPermissions(path, mode=None, uid=None, gid=None, mkdir=False):
  """Set the permissions and ownership of a path.

  Args:
    path: string, the path for which owner ID and group ID needs to be setup.
    mode: octal string, the permissions to set on the path.
    uid: int, the owner ID to be set for the path.
    gid: int, the group ID to be set for the path.
    mkdir: bool, True if the directory needs to be created.
  """
  if mkdir and not os.path.exists(path):
    os.mkdir(path, mode or 0o777)
  elif mode:
    os.chmod(path, mode)
  if uid and gid:
    os.chown(path, uid, gid)
  _SetSELinuxContext(path)


def Lock(fd, path, blocking):
  """Lock the provided file descriptor.

  Args:
    fd: int, the file descriptor of the file to lock.
    path: string, the name of the file to lock.
    blocking: bool, whether the function should return immediately.

  Raises:
    IOError, raised from flock while attempting to lock a file.
  """
  operation = fcntl.LOCK_EX if blocking else fcntl.LOCK_EX | fcntl.LOCK_NB
  try:
    fcntl.flock(fd, operation)
  except IOError as e:
    if e.errno == errno.EWOULDBLOCK:
      raise IOError('Exception locking %s. File already locked.' % path)
    else:
      raise IOError('Exception locking %s. %s.' % (path, str(e)))


def Unlock(fd, path):
  """Release the lock on the file.

  Args:
    fd: int, the file descriptor of the file to unlock.
    path: string, the name of the file to lock.

  Raises:
    IOError, raised from flock while attempting to release a file lock.
  """
  try:
    fcntl.flock(fd, fcntl.LOCK_UN | fcntl.LOCK_NB)
  except IOError as e:
    if e.errno == errno.EWOULDBLOCK:
      raise IOError('Exception unlocking %s. Locked by another process.' % path)
    else:
      raise IOError('Exception unlocking %s. %s.' % (path, str(e)))


@contextlib.contextmanager
def LockFile(path, blocking=False):
  """Interface to flock-based file locking to prevent concurrent executions.

  Args:
    path: string, the name of the file to lock.
    blocking: bool, whether the function should return immediately.

  Yields:
    None, yields when a lock on the file is obtained.

  Raises:
    IOError, raised from flock locking operations on a file.
    OSError, raised from file operations.
  """
  fd = os.open(path, os.O_CREAT)
  try:
    Lock(fd, path, blocking)
    yield
  finally:
    try:
      Unlock(fd, path)
    finally:
      os.close(fd)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for boto_config.py module."""

from google_compute_engine.boto import boto_config
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class BotoConfigTest(unittest.TestCase):

  def setUp(self):
    self.project_id = 'project'
    boto_config.BotoConfig.boto_config = 'config'
    boto_config.BotoConfig.boto_config_template = 'template'
    boto_config.BotoConfig.boto_config_script = '/tmp/test.py'
    boto_config.BotoConfig.boto_config_header = '%s %s'

  @mock.patch('google_compute_engine.boto.boto_config.metadata_watcher')
  @mock.patch('google_compute_engine.boto.boto_config.logger')
  @mock.patch('google_compute_engine.boto.boto_config.config_manager')
  def testCreateConfig(self, mock_config, mock_logger, mock_watcher):
    mock_config_instance = mock.Mock()
    mock_config.ConfigManager.return_value = mock_config_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_config.ConfigManager, 'config')
    mocks.attach_mock(mock_config_instance.SetOption, 'set')
    mocks.attach_mock(mock_config_instance.WriteConfig, 'write')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance

    boto_config.BotoConfig(self.project_id, debug=True)
    expected_calls = [
        mock.call.logger.Logger(name=mock.ANY, debug=True),
        mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
        mock.call.config(
            config_file='template', config_header='/tmp/test.py template'),
        mock.call.set('GSUtil', 'default_project_id', self.project_id),
        mock.call.set('GSUtil', 'default_api_version', '2'),
        mock.call.set('GoogleCompute', 'service_account', 'default'),
        mock.call.write(config_file='config'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.boto.boto_config.metadata_watcher')
  @mock.patch('google_compute_engine.boto.boto_config.config_manager')
  def testCreateConfigProjectId(self, mock_config, mock_watcher):
    mock_config_instance = mock.Mock()
    mock_config.ConfigManager.return_value = mock_config_instance
    mock_watcher_instance = mock.Mock()
    mock_watcher.MetadataWatcher.return_value = mock_watcher_instance
    mock_watcher_instance.GetMetadata.return_value = self.project_id

    boto_config.BotoConfig()
    mock_watcher_instance.GetMetadata.assert_called_once_with(
        metadata_key='project/numeric-project-id', recursive=False)
    expected_calls = [
        mock.call('GSUtil', 'default_project_id', self.project_id),
    ]
    mock_config_instance.SetOption.assert_has_calls(expected_calls)

  @mock.patch('google_compute_engine.boto.boto_config.metadata_watcher')
  @mock.patch('google_compute_engine.boto.boto_config.config_manager')
  def testCreateConfigExit(self, mock_config, mock_watcher):
    mock_config_instance = mock.Mock()
    mock_config.ConfigManager.return_value = mock_config_instance
    mock_watcher_instance = mock.Mock()
    mock_watcher.MetadataWatcher.return_value = mock_watcher_instance
    mock_watcher_instance.GetMetadata.return_value = None

    boto_config.BotoConfig()
    mock_watcher_instance.GetMetadata.assert_called_once_with(
        metadata_key='project/numeric-project-id', recursive=False)
    mock_config.SetOption.assert_not_called()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unittest for utils.py module."""

import subprocess

from google_compute_engine.distro_lib.sles_11 import utils
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class UtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_setup = mock.create_autospec(utils.Utils)

  def testEnableNetworkInterfacesWithSingleNic(self):
    mocks = mock.Mock()

    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['eth0'], self.mock_logger)
    expected_calls = []
    self.assertEqual(mocks.mock_calls, expected_calls)

  def testEnableNetworkInterfacesWithMultipleNics(self):
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_setup._Dhcpcd, 'dhcpcd')

    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['eth0', 'eth1', 'eth2'], self.mock_logger)
    expected_calls = [
        mock.call.dhcpcd(['eth1', 'eth2'], mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch(
      'google_compute_engine.distro_lib.sles_11.utils.subprocess.check_call')
  def testDhcpcd(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')
    mock_call.side_effect = [
        None, None, None, None,
        subprocess.CalledProcessError(1, 'Test'),
        subprocess.CalledProcessError(1, 'Test'),
    ]

    utils.Utils._Dhcpcd(
        self.mock_setup, ['eth1', 'eth2', 'eth3'], self.mock_logger)
    expected_calls = [
        mock.call.call(['/sbin/dhcpcd', '-x', 'eth1']),
        mock.call.call(['/sbin/dhcpcd', 'eth1']),
        mock.call.call(['/sbin/dhcpcd', '-x', 'eth2']),
        mock.call.call(['/sbin/dhcpcd', 'eth2']),
        mock.call.call(['/sbin/dhcpcd', '-x', 'eth3']),
        mock.call.logger.info(mock.ANY, 'eth3'),
        mock.call.call(['/sbin/dhcpcd','eth3']),
        mock.call.logger.warning(mock.ANY, 'eth3'),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallHwclock')
  def testHandleClockSync(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.HandleClockSync(self.mock_setup, self.mock_logger)
    expected_calls = [mock.call.call(mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.IpForwardingUtilsIproute')
  def testIpForwardingUtils(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.IpForwardingUtils(self.mock_setup, self.mock_logger, '66')
    expected_calls = [mock.call.call(mock.ANY, '66')]
    self.assertEqual(mocks.mock_calls, expected_calls)
